/****************** <lalVerbatim file="BlockInitSearchCV">
Author: Tibbits, M. M.
$Id: BlockInitSearch.c,v 1.18 2003/07/06 20:06:08 jolien Exp $
********************************* </lalVerbatim> */

/******************************* <lalLaTeX file="BlockInitSearchC">
\subsection{Module \texttt{BlockInitSearch.c}}
\label{s:BlockInitSearch.c}

Routine that checks the validity of the input parameters and initializes 
them.

\subsubsection*{Prototypes}
\input{BlockInitSearchCP}
\index{\texttt{LALInitSearch()}}

\subsection*{Description}

Routine that checks the validity of the input parameters and initializes
them.

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockInitSearchCV}}

*************************************************** </lalLaTeX> */

#include "Block.h"

NRCSID( BLOCKINITSEARCHC, "$Id: BlockInitSearch.c,v 1.18 2003/07/06 20:06:08 jolien Exp $");

/************************ <lalVerbatim file="BlockInitSearchCP"> */
void LALInitSearch
(
	LALStatus		*status,
	void			**searchParams,
	LALInitSearchParams	*initSearchParams
)
/*********************************************** </lalVerbatim> */
{
	BlockSearchParams	*local;
	INT4			bands;

	INITSTATUS( status, "LALInitSearch", BLOCKINITSEARCHC );

	local = LALMalloc(sizeof(BlockSearchParams));
	if(!local)
	{
		ABORT( status, BLOCKH_EALOC, BLOCKH_MSGEALOC );
	}

	/*  Turn on Debugging 
	lalDebugLevel = LALALLDBG;
*/
	/*  Initialize parameters  */
	local->searchMaster	= initSearchParams->nodeClass;
	local->rank		= 0;

	local->numberOfSequences	= 0;
	local->segmentsRemaining	= 0;
	local->resultsObtained		= 0;
	local->numSlaves		= 0;
	local->numOutput		= 0;

	local->lastResultStored		= NULL;
	local->resultsToStore		= NULL;
	local->eventCounter		= 0;
	local->prevResults		= NULL;
	local->results			= NULL;
	local->head			= NULL;
	local->tail			= NULL;
	local->data			= NULL;

	strcpy(local->channel, "");
	strcpy(local->ifo, "");

	ASSERT( searchParams, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	ASSERT( initSearchParams, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	ASSERT( initSearchParams->argv, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	/*  Allows for more than expect params  */
	ASSERT( initSearchParams->argc >= 15, status, BLOCKH_EARG, BLOCKH_MSGEARG );

	ASSERT(!(strcmp( initSearchParams->argv[0], "-filterparams" )), status, BLOCKH_EARG, BLOCKH_MSGEARG);

	local->maxVectorLength	 = atoi( initSearchParams->argv[2] );
	local->minVectorLength	 = atoi( initSearchParams->argv[3] );

	local->scaleFactor	 = atof( initSearchParams->argv[4] );
	local->scaleExponent	 = atof( initSearchParams->argv[5] );

	local->overlap		 = atof( initSearchParams->argv[6] );
	local->overlapMethod	 = atoi( initSearchParams->argv[7] );

	local->marginOfExclusion = atoi( initSearchParams->argv[8] );

	strcpy(local->channel, initSearchParams->argv[9]);

	bands = atoi ( initSearchParams->argv[10] );

	local->sampleRate	 = (INT4*)(LALMalloc( sizeof(INT4) * bands ));
	local->threshold 	 = (REAL8*)(LALMalloc( sizeof(REAL8) * bands ));
	local->meanOffset	 = (REAL8*)(LALMalloc( sizeof(REAL8) * bands ));
	local->meanThreshold	 = (REAL8*)(LALMalloc( sizeof(REAL8) * bands ));
	local->varianceThreshold = (REAL8*)(LALMalloc( sizeof(REAL8) * bands ));

	switch( bands )
	  {
	  case 1:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le",&(local->threshold[0]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d",&(local->sampleRate[0]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le",&(local->meanOffset[0]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le",&(local->meanThreshold[0]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le",&(local->varianceThreshold[0]));
	    break;
	  case 2:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le,%le",
		    &(local->threshold[0]),
		    &(local->threshold[1]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d,%d",
		    &(local->sampleRate[0]),
		    &(local->sampleRate[1]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le,%le",
		    &(local->meanOffset[0]),
		    &(local->meanOffset[1]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le,%le",
		    &(local->meanThreshold[0]),
		    &(local->meanThreshold[1]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le,%le",
		    &(local->varianceThreshold[0]),
		    &(local->varianceThreshold[1]));
	    break;
	  case 3:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le,%le,%le",
		    &(local->threshold[0]),
		    &(local->threshold[1]),
		    &(local->threshold[2]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d,%d,%d",
		    &(local->sampleRate[0]),
		    &(local->sampleRate[1]),
		    &(local->sampleRate[2]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le,%le,%le",
		    &(local->meanOffset[0]),
		    &(local->meanOffset[1]),
		    &(local->meanOffset[2]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le,%le,%le",
		    &(local->meanThreshold[0]),
		    &(local->meanThreshold[1]),
		    &(local->meanThreshold[2]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le,%le,%le",
		    &(local->varianceThreshold[0]),
		    &(local->varianceThreshold[1]),
		    &(local->varianceThreshold[2]));
	    break;
	  case 4:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le,%le,%le,%le",
		    &(local->threshold[0]),
		    &(local->threshold[1]),
		    &(local->threshold[2]),
		    &(local->threshold[3]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d,%d,%d,%d",
		    &(local->sampleRate[0]),
		    &(local->sampleRate[1]),
		    &(local->sampleRate[2]),
		    &(local->sampleRate[3]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le,%le,%le,%le",
		    &(local->meanOffset[0]),
		    &(local->meanOffset[1]),
		    &(local->meanOffset[2]),
		    &(local->meanOffset[3]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le,%le,%le,%le",
		    &(local->meanThreshold[0]),
		    &(local->meanThreshold[1]),
		    &(local->meanThreshold[2]),
		    &(local->meanThreshold[3]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le,%le,%le,%le",
		    &(local->varianceThreshold[0]),
		    &(local->varianceThreshold[1]),
		    &(local->varianceThreshold[2]),
		    &(local->varianceThreshold[3]));
	    break;
	  case 5:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le,%le,%le,%le,%le",
		    &(local->threshold[0]),
		    &(local->threshold[1]),
		    &(local->threshold[2]),
		    &(local->threshold[3]),
		    &(local->threshold[4]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d,%d,%d,%d,%d",
		    &(local->sampleRate[0]),
		    &(local->sampleRate[1]),
		    &(local->sampleRate[2]),
		    &(local->sampleRate[3]),
		    &(local->sampleRate[4]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le,%le,%le,%le,%le",
		    &(local->meanOffset[0]),
		    &(local->meanOffset[1]),
		    &(local->meanOffset[2]),
		    &(local->meanOffset[3]),
		    &(local->meanOffset[4]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le,%le,%le,%le,%le",
		    &(local->meanThreshold[0]),
		    &(local->meanThreshold[1]),
		    &(local->meanThreshold[2]),
		    &(local->meanThreshold[3]),
		    &(local->meanThreshold[4]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le,%le,%le,%le,%le",
		    &(local->varianceThreshold[0]),
		    &(local->varianceThreshold[1]),
		    &(local->varianceThreshold[2]),
		    &(local->varianceThreshold[3]),
		    &(local->varianceThreshold[4]));
	    break;
	  case 6:
	    sscanf( ((CHAR*)(initSearchParams->argv[1])),"%le,%le,%le,%le,%le,%le",
		    &(local->threshold[0]),
		    &(local->threshold[1]),
		    &(local->threshold[2]),
		    &(local->threshold[3]),
		    &(local->threshold[4]),
		    &(local->threshold[5]));
	    sscanf( ((CHAR*)(initSearchParams->argv[11])),"%d,%d,%d,%d,%d,%d",
		    &(local->sampleRate[0]),
		    &(local->sampleRate[1]),
		    &(local->sampleRate[2]),
		    &(local->sampleRate[3]),
		    &(local->sampleRate[4]),
		    &(local->sampleRate[5]));
	    sscanf( ((CHAR*)(initSearchParams->argv[12])),"%le,%le,%le,%le,%le,%le",
		    &(local->meanOffset[0]),
		    &(local->meanOffset[1]),
		    &(local->meanOffset[2]),
		    &(local->meanOffset[3]),
		    &(local->meanOffset[4]),
		    &(local->meanOffset[5]));
	    sscanf( ((CHAR*)(initSearchParams->argv[13])),"%le,%le,%le,%le,%le,%le",
		    &(local->meanThreshold[0]),
		    &(local->meanThreshold[1]),
		    &(local->meanThreshold[2]),
		    &(local->meanThreshold[3]),
		    &(local->meanThreshold[4]),
		    &(local->meanThreshold[5]));
	    sscanf( ((CHAR*)(initSearchParams->argv[14])),"%le,%le,%le,%le,%le,%le",
		    &(local->varianceThreshold[0]),
		    &(local->varianceThreshold[1]),
		    &(local->varianceThreshold[2]),
		    &(local->varianceThreshold[3]),
		    &(local->varianceThreshold[4]),
		    &(local->varianceThreshold[5]));
	    break;
 
	  default:
	    ABORT( status, BLOCKH_EARG, BLOCKH_MSGEARG);
	  }

	strncpy(local->ifo, initSearchParams->argv[9],2);
	local->ifo[2] = '\0';

	*searchParams = local;
	RETURN(status);
}
