/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigBuildOutputCV"> ********
Author: Rahkola, R
$Id: ExtTrigBuildOutput.c,v 1.14 2003/07/03 18:30:56 jolien Exp $
********* </lalVerbatim> ********/

/* standard C functions */
#include <math.h>
#include <stdio.h>
#include <string.h>

/* standard LAL functions*/
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/Units.h>
#include <lal/Date.h>

/* LAL MPI interface */
#include <lal/Comm.h>

/* Associated headers */
#include <ExtTrig.h>
#include <ExtTrigApplySearch.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGBUILDOUTPUTC, "$Id: ExtTrigBuildOutput.c,v 1.14 2003/07/03 18:30:56 jolien Exp $" );

/********************** local static function definitions ********************/

/*
 * Function ExtTrigBuildDB
 *
 * Create a linked list containing the database entries and put it in
 * the search output structure
 */
void
ExtTrigBuildDB(
	       LALStatus             *const status,
	       LALSearchOutput       *const output,
	       const ETSearchParams  *const params )
{
  /*
   * The program name which appears in the PROGRAM field of
   * the SUMM_STATISTICS table (not sure if I need this yet)
   */
  /* const CHAR             *const PROGRAM = "ExtTrig"; */
  dataBase               *prev          = 0; /* pointer to previous db link */
  dataBase               *dbColumn      = 0; /* pointer to current db link  */
  UINT4                   dbLength      = params->result->length;
  outPut                 *searchOut     =        /* search_summary results  */
    output->result;
  outPut                 *outputArray   =        /* exttrig search results  */
    &(output->result[1]);
  UINT4                   i, slength;  /* iterator(s), string length */
  /* strings to be placed in the database; some will vary for on/off-source */
  /* the lengths are determined by the multi_burst table definition         */
  CHAR *filter_on, *filter_off, *ifos, *event_on, *event_off, *event_type;
  CHAR tmp[4]; /* temporary placeholder for event_* strings */
  CHAR                    searchName[]  = "exttrig";
  
  INITSTATUS( status, "BuildOutput", EXTTRIGBUILDOUTPUTC );
  ATTATCHSTATUSPTR( status );

#ifdef DEBUG_EXTTRIG
  fprintf( stdout, "exttrig:ExtTrigBuildOutput - %s\n", params->id );
  fflush ( stdout );
#endif

  /*
   *
   * setup output strings: filter_on/off, ifos, event_type & event_on/off
   * :NOTE: filter_id and event_id will be auto-filled
   *
   */
  /* first the event_id strings */
  event_on   = (CHAR *) LALCalloc( 13, sizeof(CHAR) );
  event_off  = (CHAR *) LALCalloc( 13, sizeof(CHAR) );
  sprintf( event_on,  "GRB  on-src" );
  sprintf( event_off, "GRB off-src" );
  /* next the event_type string */
  event_type = (CHAR *) LALCalloc( 32, sizeof(CHAR) );
  sprintf( event_type, "GRB" );
  /* next the ifos string */
  ifos       = (CHAR *) LALCalloc( 12, sizeof(CHAR) );
  strncat( ifos, params->chan1, 2 );
  strcat ( ifos, " " );
  strncat( ifos, params->chan2, 2 );
  /* finally the filter_id strings */
  filter_on  = (CHAR *) LALCalloc( 13, sizeof(CHAR) );
  filter_off = (CHAR *) LALCalloc( 13, sizeof(CHAR) );
  sprintf( filter_on,  EXTTRIG_VER );
  sprintf( filter_off, EXTTRIG_VER );
  /* :TODO: It would be nice to somehow put in the job number in the event_id*/
  strcat ( filter_on,  ",j000" );
  strcat ( filter_off, ",j000" );
  sprintf( tmp, ",%u", params->initParams->initMask      ); /* off-source */
  strcat ( filter_off, tmp );
  sprintf( tmp, ",%u", (params->initParams->initMask +1) ); /* on-source  */
  strcat ( filter_on,  tmp );


  /*
   * Check for memory allocation errors.  Memory should be allocated in
   * LALApplySearch(Master|Slave).
   */
  if ( !searchOut ) { 
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }


  /*
   *
   * create the search_summary database entry
   *
   */
  searchOut->indexNumber    = 0;
  searchOut->search         = burst;
  searchOut->significant    = 1;
  searchOut->states         = NULL;
  searchOut->results        = NULL;
  searchOut->optional       = NULL;
  params->info->dsoName       = searchName;
  LALCreateSearchSummaryDataBase( status->statusPtr, &(searchOut->results), 
				  params->info );
  CHECKSTATUSPTR( status );

  /*
   * initialize the output array : atomic datatypes
   *
   */
  outputArray = &(output->result[1]);
  outputArray->indexNumber  = 1;
  outputArray->search       = burst;
  outputArray->significant  = 1;
  outputArray->states       = NULL;
  outputArray->results      = NULL;
  outputArray->optional     = NULL;


  /*
   * initialize the output array : database linked list (by columns)
   * :NOTE: Since the exttrig produces multiple statistics for each GRB trigger
   * :NOTE:we can make use of the "numberRows" feature of the dataBase linked
   * :NOTE:list.  Thus in what follows, we'll always be allocating "dbLength"
   * :NOTE:number of entries, and filling them in one at a time.
   * :NOTE: Alternatively, we could have set output->numOutput to the number of
   * :NOTE:rows we want to enter and always keep "numberRows"=1.  I feel this
   * :NOTE:method, although it involves more for loops, requires less time in
   * :NOTE:memory allocation, and is more representative of the fact that all
   * :NOTE:rows are related to the same external trigger event.
   *
   */
  dbColumn = 
    outputArray->results = (dataBase *) LALCalloc( 1, sizeof(dataBase) ); 
  if ( !dbColumn ) {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC ); 
  } 
  dbColumn->previous = NULL;

/*
 *
 * INTERFEROMETER INFORMATION
 *
 */
  /*
   * 1st column: ifos (non-null char[12]) = "H1 H2" | "H1 L1" | "H2 L1" etc.
   */
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "ifos" );
  dbColumn->type          = char_s_ptr;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = (UINT4 *) LALMalloc( dbLength * sizeof(UINT4) );
  slength = strlen(ifos);
  dbColumn->rows.chars    =
    (CHAR *) LALCalloc( dbLength * slength, sizeof(CHAR) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rowDimensions[i] = slength;
    strncpy( dbColumn->rows.chars + i*slength, ifos, slength * sizeof(CHAR) );
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

/*
 *
 * EXTERNAL TRIGGER INFORMATION
 *
 */
  /*
   * 2nd column: exttrig_type (non-null char[32]) = "GRB" (default), or 
   *                 whatever is obtained vi metadata query
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "exttrig_type" );
  dbColumn->type          = char_s_ptr;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = (UINT4 *) LALMalloc( dbLength * sizeof(UINT4) );
  slength = strlen(ifos);
  dbColumn->rows.chars    =
    (CHAR *) LALCalloc( dbLength * slength, sizeof(CHAR) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rowDimensions[i] = slength;
    strncpy( dbColumn->rows.chars + i*slength,
	     event_type, slength * sizeof(CHAR) );
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 3rd column: exttrig_id (char[40]) = event_id from the metadata query
   */
/*
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "exttrig_id" );
  dbColumn->type          = char_s_ptr;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = (UINT4 *) LALMalloc( dbLength * sizeof(UINT4) );
  slength = strlen(ifos);
  dbColumn->rows.chars    =
    (CHAR *) LALCalloc( dbLength * slength, sizeof(CHAR) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rowDimensions[i] = slength;
    strncpy( dbColumn->rows.chars + i*slength, ifos, slength * sizeof(CHAR) );
  }
  prev = dbColumn;
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );
*/

  /*
   * 4th column: start_time (non-null INT4) = start time of external trigger
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "start_time" );
  dbColumn->type          = int_4s;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.int4s    = (INT4 *) LALCalloc( dbLength, sizeof(INT4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.int4s[i]    = params->initParams->grbStart.gpsSeconds;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 5th column: start_time_ns (non-null INT4) = start time (ns) of ext trigger
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "start_time_ns" );
  dbColumn->type          = int_4s;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.int4s    = (INT4 *) LALCalloc( dbLength, sizeof(INT4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.int4s[i]    = params->initParams->grbStart.gpsNanoSeconds;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 6th column: source_ra (REAL4) = source right ascension angle
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "source_ra" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i] = (REAL4) params->initParams->sourceDir.latitude;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 7th column: source_ra_err (REAL4) = error in source right ascension
   * :TODO: Fill source_ra_err column
   */

  /*
   * 8th column: source_dec (REAL4) = source declination angle
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "source_dec" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i] = (REAL4) params->initParams->sourceDir.longitude;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 9th column: source_dec_err (REAL4) = error in source declination
   * :TODO: Fill source_dec_err column
   */

/*
 *
 * SEGMENT INFORMATION
 *
 */
  /*
   * 10th column: segment_type (INT4) = "1" for on-source, "0" for off-source
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "segment_type" );
  dbColumn->type          = int_4s;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.int4s    = (INT4 *) LALCalloc( dbLength, sizeof(INT4) );
  for ( i = 0; i < dbLength; i++ ) {
    if ( !i ) dbColumn->rows.int4s[i] = 1;
    else      dbColumn->rows.int4s[i] = 0;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );


  /*
   * 11th column: duration (non-null REAL4) = segment length (seconds)
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "duration" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i] = 
      ((REAL4) params->initParams->segLength) / params->initParams->sampleRate;
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 12th column: ref_delay (REAL4) = delay from exttrig start to
   *                                  start of IFO1 segment
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "ref_delay" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i]    = params->result->xCorrDelay[i];
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 13th column: lag_1_2 (REAL4) = delay from start of IFO1 segment to
   *                                start of IFO2 segment
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "lag_1_2" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i]    = params->result->xCorrLag[i];
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 14th column: lag_1_3 (REAL4) = delay from start of IFO1 segment to
   *                                start of IFO3 segment
   * :TODO: Fill lag_1_3 column
   */

  /*
   * 15th column: lag_1_4 (REAL4) = delay from start of IFO1 segment to
   *                                start of IFO4 segment
   * :TODO: Fill lag_1_4 column
   */

/*
 *
 * RESULTS OF THE CALCULATION
 *
 */
  /*
   * 16th column: amplitude (non-null REAL4) = cross-correlation statistic
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "amplitude" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i]    = params->result->xCorrValue[i];
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 17th column: snr (REAL4) = blank for now
   * :TODO: Fill snr column
   */

  /*
   * 18th column: percent_bad (REAL4) = % of bad data used in segment
   * :TODO: Fill percent_bad column
   */
/*
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "confidence" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    if( !i ) dbColumn->rows.real4[i] = params->result->badSeconds;
    / * :NOTE: This should change if we allow off-src segments to cross
       :NOTE:up to some threshold of bad data points.
     * /
    else     dbColumn->rows.real4[i] = 0;
  }
  prev = dbColumn;
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );
*/

  /*
   * 19th column: var_1 (REAL4) = auto-correlation, channel 1
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "var_1" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i]    = params->result->variance1[i];
  }
  prev = dbColumn; /* move to next column */
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );

  /*
   * 20th column: var_2 (REAL4) = auto-correlation, channel 2
   */
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "var_2" );
  dbColumn->type          = real_4;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = NULL;
  dbColumn->rows.real4    = (REAL4 *) LALCalloc( dbLength, sizeof(REAL4) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rows.real4[i]    = params->result->variance2[i];
  }

  /*
   * 21st column: var_3 (REAL4) = auto-correlation, channel 3
   * :TODO: Fill var_3 column
   */

  /*
   * 22nd column: var_4 (REAL4) = auto-correlation, channel 4
   * :TODO: Fill var_4 column
   */

/*
  prev = dbColumn;
  dbColumn = prev->next = (dataBase *) LALCalloc( 1, sizeof(dataBase) );
*/
  /*
   * 21st column: event_id (non-null CHAR[13]) = ???
   */
/* :NOTE: filter_id and event_id are auto-filled
  dbColumn->previous      = prev;
  sprintf( dbColumn->tableName,  "exttrig_search" );
  sprintf( dbColumn->columnName, "event_id" );
  dbColumn->type          = char_s_ptr;
  dbColumn->numberRows    = dbLength;
  dbColumn->rowDimensions = (UINT4 *) LALMalloc( dbLength * sizeof(UINT4) );
  slength = strlen(event_off);
  dbColumn->rows.chars    =
    (CHAR *) LALCalloc( dbLength * slength, sizeof(CHAR) );
  for ( i = 0; i < dbLength; i++ ) {
    dbColumn->rowDimensions[i] = slength;
    if ( !i ) strncpy( dbColumn->rows.chars + i * slength, 
		       event_on,   slength * sizeof(CHAR) );
    else      strncpy( dbColumn->rows.chars + i * slength, 
		       event_off,  slength * sizeof(CHAR) );
  }
*/
  dbColumn->next = NULL;
      
  /*
   * Free temporary allocated memory
   */
  LALFree( filter_off );
  LALFree( filter_on );
  LALFree( ifos );
  LALFree( event_type );
  LALFree( event_off );
  LALFree( event_on );

  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}


/*
 * Function ExtTrigBuildILWD
 *
 * Create/Append output data to the search output structure.
 * Each slave calls ExtTrigBuildILWD for each spectra which is created, once
 * the master tells it to stop applying the search method. The function below
 * allocates space for an additional outPut structure appended to
 * output->result.  Only the multiDimData structure 'optional' is used; the
 * dataBase structure 'results' is set to null.
 *
 * :NOTE: ExtTrigBuildILWD should be called after all ExtTrigBuildOutput calls
 * :NOTE: since the latter does not look to see if output->result has already
 * :NOTE: been allocated.
 *
 * Calling parameters:
 * 'status' - the LALStatus object which can thread ABORT statements
 * 'output' - the outPut structure for the calling node (may contain previous
 *            output already)
 * 'index'  - the index of ETResults.xCorrSpectra to translate to multiDimData
 * 'params' - the entire ETSearchParams tree to be used as reference (_not_ to
 *            be modified!)
 *
 */
void
ExtTrigBuildILWD(
		 LALStatus               *const status,
		 LALSearchOutput         *const output,
		 const ETSearchParams    *const params )
{
  /* const CHAR             *const PROGRAM = "ExtTrig";*/
  UINT4                   startOut      = 0;          /* first empty outPut */
  UINT4                   i;             /* used to find first empty outPut */
  multiDimData           *mddEntry      = 0;
  outPut                 *outputArray;
  COMPLEX8FrequencySeries **spectra     = params->result->xCorrSpectra;
  CHARVector             *vchar;      /* Used in LALUnitAsString */
  REAL8                   endREAL8;   /* Used to determine end time */
  LIGOTimeGPS             endGPS;     /* Used to determine end time */
  
  INITSTATUS( status, "ExtTrigBuildILWD", EXTTRIGBUILDOUTPUTC );
  ATTATCHSTATUSPTR( status );

#ifdef DEBUG_EXTTRIG
  fprintf( stdout, "exttrig:ExtTrigBuildILWD - %s\n", params->id );
  fflush ( stdout );
#endif

  /*
   * Create the "units" string, using LALUnitAsString
   */
  LALCHARCreateVector( status->statusPtr, &vchar, LALUnitTextSize );
  CHECKSTATUSPTR( status );
  LALUnitAsString( status->statusPtr, vchar, &(spectra[0]->sampleUnits) );
  CHECKSTATUSPTR( status );
  
  /*
   * Check for memory allocation errors.  Memory should be allocated in
   * LALApplySearch(Master|Slave).
   */
  if ( !(output->result) ) { 
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }


  /*
   * initialize the output array : atomic datatypes & memory allocation
   * BEWARE!  Can't assume that we are starting with a blank results!
   * However, we should be able to assume that the result->optional is not
   * yet allocated.
   */

  /* All results are allocated in ExtTrigApplySearch[Master/Slave]; here we
   * find the first empty result available for ILWD output, and assume enough
   * are allocated to contain all the desired spectra.  Reallocating the memory
   * is not good enough, because it's possible some pointers may be lost in the
   * process.
   */
  while ( output->result[startOut].results ||
	  output->result[startOut].optional  ) startOut++;

  /* Iterate through the params->result->xCorrSpectra and layout the ILWD*/
  for ( i = 0; i < params->result->filled; i++ ) {
    outputArray = &(output->result[startOut + i]);

    outputArray->indexNumber  = startOut;
    outputArray->search       = burst;
    outputArray->significant  = 1;
    outputArray->states       = NULL;
    outputArray->results      = NULL;
    outputArray->optional     = NULL;
    mddEntry = outputArray->optional = 
      (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
    if ( !mddEntry ) { 
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }
#if DEBUG_EXTTRIG > 1
    fprintf( stdout, "%s: output->result:%p,\tindex:%d,\toutputArray:%p\n",
	     params->id, output->result, i, outputArray );
    fflush( stdout );
#endif /*  DEBUG_EXTTRIG > 1*/

    /*
     * fill the multiDimData array
     */
    /* Compute the end time for the frequency series */
    LALGPStoFloat( status->statusPtr, &endREAL8, &(spectra[i]->epoch) );
    CHECKSTATUSPTR( status );
    endREAL8 += spectra[i]->data->length *  spectra[i]->deltaF;
    LALFloatToGPS( status->statusPtr, &endGPS, &endREAL8 );
    CHECKSTATUSPTR( status );

    /* CHAR        name[maxMultiDimName] */
    sprintf( mddEntry->name, spectra[i]->name );
    /* CHAR        units[maxMultiDimUnits] */
    strncpy( mddEntry->units, vchar->data, maxMultiDimUnits );
    /* domain      space */
    mddEntry->space      = freqD;
    /* datatype    type */
    mddEntry->type       = complex_8;
    /* interval    range :: complex datatype */
    mddEntry->range.dTime.numberSamples =  /* UINT8 numberSamples */
      spectra[i]->data->length;
    mddEntry->range.dTime.startSec =  /* UINT4 startsec */
      spectra[i]->epoch.gpsSeconds;
    mddEntry->range.dTime.startNan =  /* UINT4 startNan */
      spectra[i]->epoch.gpsNanoSeconds;
    mddEntry->range.dTime.stopSec =   /* UINT4 stopSec */
      endGPS.gpsSeconds;
    mddEntry->range.dTime.stopNan =   /* UINT4 stopNan */
      endGPS.gpsNanoSeconds;
    mddEntry->range.dTime.timeStepSize = /* REAL8 timeStepSize -- non-null */
      spectra[i]->deltaF;
    mddEntry->range.dTime.baseFreq =  /* REAL8 baseFreq */
      spectra[i]->f0;
    mddEntry->range.dTime.phase = 0; /* REAL8 phase */
    /* UINT4       numberDimensions */
    mddEntry->numberDimensions = 1; /* correspond to ccStatistic & offset */
    /* UINT4*      dimensions */
    mddEntry->dimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    if ( !(mddEntry->dimensions) ) {
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }
    /* :TODO: add variances as data */
    *(mddEntry->dimensions) = spectra[i]->data->length;
    /* dcHistory*  history */
    mddEntry->history    = NULL;
    /* dataPointer data */
    /* :TODO: add 2 to the size to include variances */
    mddEntry->data.complex8 = (COMPLEX8 *) 
      LALCalloc ( spectra[i]->data->length, sizeof(COMPLEX8) );
    if ( !(mddEntry->data.complex8) ) {
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }

    /*
     * Copy the data to multiDimData structure
     *
     */
    memcpy( mddEntry->data.complex8, spectra[i]->data->data,
	    spectra[i]->data->length * sizeof(COMPLEX8) );
  } /*  for ( i = 0; i < params->result->filled; i++ ) ...*/

  /*
   * Free allocated memory
   */
  LALCHARDestroyVector( status->statusPtr, &vchar );
  CHECKSTATUSPTR( status );

  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}

#ifdef __cplusplus
}
#endif
