/******** <lalVerbatim file="FCTHV"> ********
Author : Wen, L.
$ID$
********* </lalVerbatim> ********/

#ifndef _FCT_H
#define _FCT_H

#include <stdio.h> 
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <math.h>

#include <lal/LALStdlib.h> 
#include <lal/SeqFactories.h>
#include <lal/LALRCSID.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALConstants.h>
#include <lal/LALDatatypes.h>

#include <lal/LALFCTInterface.h>

#include "LALWrapperInterface.h"
#include "FCTMyPhaseFunc.h" /*  user supplied if #func>4 or if there is a name change */

/*  define this for verbose debugging messages  */
   #undef FCT_SO_DEBUG_DATA     
 #undef FCT_SO_DEBUG 

/* #define FCT_SO_DEBUG_DATA 1   */

/*   #define FCT_SO_DEBUG 1    */

/*  define these for debugging utils in the shared object */
/*     #define FCT_SO_ATTACH_DDD  1 */
#undef FCT_SO_ATTACH_DDD 

#undef FCT_SO_GRAPHING_FUNCS


#ifdef  __cplusplus   /* C++ protection. */
extern "C" {
#endif


NRCSID( FCTGENERALH, "$ID$");

/******** <lalErrTable file="FCTHErrTab"> ********/
#define FCTGENERALH_ENULL 1
#define FCTGENERALH_EMEM  2
#define FCTGENERALH_ENULLE 3
#define FCTGENERALH_MSGEMEM  "Out of memory."
#define FCTGENERALH_MSGENULL "Null pointer."
#define FCTGENERALH_MSGENULLE "No Events."
/******** </lalErrTable> ********/

  /* some constants, in cgs units */

#define  f0 40.0L /*  same definition as Sathya's, specific to Inspiral */
#define  fCut 25.0L /* low frequency cut-off, consistent with Tev's code */

  /* need to be replaced */
  /*  #define  LALPI  3.1415926  */
  /* #define  fa 40.0L  */ /* = f0 at FCTMyPhaseFunc, check and replace fa with f0 */
  /* #define  Ts 4.92549095e-6 */ /* GMs/c^3 */
  /* #define  RsSun 2.9535e5 */  /* schwarzschild radius of the sun */  

#define  Mpc 3.0856775807e22  /* SI unit */   
#define  powInd 1.166666666667 /*  f^(-7/6) -dependence in amplitude */
#define  incl   0.0      /* in rad, fixed */


/* structure to store statistical information for imaginary numbers*/
typedef struct
tagFCTMomentType 
{
  COMPLEX8 avg;   
  COMPLEX8 sigma2;
}FCTMomentType;


typedef struct
tagFCTEventType
{
  LIGOTimeGPS                      time;
  REAL4                            effDist;
  REAL4                            mass1; 
  REAL4                            mass2;
  REAL4                            eta;
  REAL4                            t0; 
  REAL4                            t2; 
  REAL4                            t3; 
  REAL4                            t4; 
  REAL4                            tC; 
  REAL4                            snr;
  REAL4                            chisq;
  REAL8                            sigma;
  struct tagFCTEventType           *nextEvent;
}FCTEventType;

typedef struct
tagFCTInParamType 
{
  INT4                          dataLength;  /*  initialized in InitSearch()*/
  INT4                          dimension;    
  INT4                          numDataCube;
  INT4                          numEvents;
  LIGOTimeGPS                      epoch;  /* to be assigned in ConditionData */ 
  REAL4                           maxF; /* maximum frequency, FCTMyInter.c, served in FCTMyPhaseFunc.c  */
  REAL8                          dt;
  REAL4                          etaNorm; /* norm to calculate eta using t0 and t3, assigned in InitSearch */
  REAL4                          totalMassNorm; /* norm to calculate total mass using t0 and t3 */
  REAL8                          distNorm;  /* partly initialized in InitSearch, calculated in ConditionData */
  REAL4                            prob;
  CHAR                          *ifoName;  /* name of the detector */

  FCTMomentType                  *freqMoment; /* variance of h(f)/S(f), real and imaginary part*/
  INT4                          *ofac;
  COMPLEX8Vector                *fctInput;  
  LALFCTPlan                    *fctPlan;
  LALFCTDataCube                *cube;   /* Philip has to add * ofac to it */
}FCTInParamType;  


/* local structures required  for LAL FCT  Interface initialization */

typedef struct
tagFCTInStrucType 
{
  LALCreateFCTPlanInput  planIn;
  LALFCTSetUnitsInput    setUnitsIn;
  LALFCTPhaseFn          phaseFuncArray; /* Check pointer array to functions, global */
  INT4                   maxFFTSeg;
}FCTInStrucType;    

/* Hier event structure */
typedef struct 
tagFCTHierEventType
{ 
  REAL4                         *location; /* Array of size dimension */
  REAL4                         power;
  struct tagFCTHierEventType    *nextEvent;
}FCTHierEventType;


typedef struct
tagFCTSearchParams /* initialized in InitSearch() */
{
  BOOLEAN                       searchMaster;
  BOOLEAN                       firstTime; /*  need this? */
  UINT4                        *numSlaves;
  UINT4                           dimension;
  INT4                          *startIndices;
  INT4                          *endIndices;
  INT4                          *indicesLength;
  INT4                          *segLengthPerRun;
  INT4                          *numSegPerDim;
  INT4                          totalNumSeg;
  INT4                          segmentCompleted;
  REAL4                         minMass;
  REAL4                         maxMass;
  FCTInParamType                *fctInParams;
  FCTHierEventType              *curEvent;
  FCTHierEventType              **hierLevelEvents; /* Array of pointers to the first event in each Hier level.*/
}FCTSearchParams;



void
LALBuildFCTOutput (
		LALStatus          *status,
		LALSearchOutput    *output,
		INT4                numEvents,
		FCTEventType             *event,
		CHAR                *ifoName

    );



void
LALMyFCTInitPlan(LALStatus *status, FCTInStrucType *fctInStrucs, LALFCTPlan  **fctPlan);

void 
LALMyFCTInterf(LALStatus *status, FCTEventType **event, FCTInParamType *fctInParams);

/*
void
LALFCTInitEvent(LALStatus *status, FCTEventType **event);
*/

void
LALFCTBuildEvent(LALStatus *status, COMPLEX8Vector *fctOut, FCTEventType **event,FCTInParamType *fctInParams);


  /* table explanation */

  /*
  IFO	  CHARACTER	2	819		N		
  SEARCH	VARCHAR	24	819		N		
  END_TIME	INTEGER	4	0		N		
  END_TIME_NS	INTEGER	4	0		N		
  IMPULSE_TIME	INTEGER	4	0		Y		
  IMPULSE_TIME_NS INTEGER	4	0	Y		
  AMPLITUDE	REAL	4	0		N		
  EFF_DISTANCE	REAL	4	0		Y		
  COA_PHASE	REAL	4	0		Y		
  MASS1	REAL	4	0		Y		
  MASS2	REAL	4	0		Y		
  MCHIRP	REAL	4	0		Y		
  ETA	REAL	4	0		Y		
  TAU0	REAL	4	0		Y		
  TAU2	REAL	4	0		Y		
  TAU3	REAL	4	0		Y		
  TAU4	REAL	4	0		Y		
  TAU5	REAL	4	0		Y		
  TTOTAL	REAL	4	0		Y		
  SNR	REAL	4	0		Y		
  CHISQ	REAL	4	0		Y		
  CHISQ_DOF	INTEGER	4	0		Y		
  SIGMASQ	DOUBLE	8	0		Y		
  */


void
LALFCTMoment(LALStatus *status, INT4 length, COMPLEX8 *dataIn, FCTMomentType *moment);

/* structures used also in Hier code */

void 
SetInitialEvents(LALStatus *status, FCTSearchParams *mySearchParams);

FCTHierEventType* 
createHierEvent(INT4 numDims);

void 
destroyEventList(FCTHierEventType* event);

REAL4 
tau2MEFunc(REAL4 m, REAL4 eta) ;

REAL4 
tau4MEFunc(REAL4 m, REAL4 eta);

REAL4 
totalMassFunc(REAL4 tau0, REAL4 tau3);

REAL4 
etaFunc(REAL4 tau0, REAL4 tau3);

void 
ChirpTimes( REAL4 *tau, UINT4 dim, REAL4 m1, REAL4 m2 );

REAL4
minFunc(REAL4 x, REAL4 y);

REAL4 
maxFunc(REAL4 x, REAL4 y);

#ifdef  __cplusplus
}
#endif  /* C++ protection. */


#endif /* _FCT_H */










