/********************************** <lalVerbatim file="PowerInitSearchCV">
Author: Brady, P and Brown, D
$Id: BuildOutput.c,v 1.22 2003/06/23 18:54:00 jolien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/ExcessPower.h>
#include <lal/TFTransform.h>
#include <lal/BurstSearch.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/EPData.h>
#include <lal/LALConstants.h>
#include "ApplySearch.h"

/*#define POWER_SO_ATTACH_DDD */

NRCSID( BUILDPOWEROUTPUTC, "power $Id: BuildOutput.c,v 1.22 2003/06/23 18:54:00 jolien Exp $");

void
BuildPowerOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    BurstEventList     *burstEventList,
    PowerOutPutParams  *params
                    )
{
  BurstEvent           *event;               /* dummy pointer */
  outPut               *outputArray;         /* dummy pointer */
  dataBase             *prev;                /* dummy pointer */
  dataBase             *dbEntry;             /* dummy pointer */
  INT4                 i;                    /* general index */
  UINT4                stringLength=0;       /* to deal with BLOBs */
  BurstEvent           *burstEvent=NULL;

  INITSTATUS (status, "BuildPowerOutPut", BUILDPOWEROUTPUTC);

  output->numOutput = 1;
  burstEvent = burstEventList->burstEvent;

  /*
   *
   * print the list events as recieved by the master
   *
   */


#ifdef POWER_SO_DBGLVL3
  {
    INT4 j=0;
    event = burstEvent;

    do
    {

      fprintf( stdout, "build output: numEvents %d\n", params->numEvents);
      fflush( stdout );
      fprintf( stdout, "build output: confidence = %e\n", event->confidence );
      fflush( stdout );

      event = event->nextEvent;
      j++;

    }while( j<params->numEvents);
  }
#endif


  /*
   *
   * create memory for the outPut array
   *
   */

  outputArray = output->result = (outPut *) LALMalloc( sizeof(outPut) );


  /*
   *
   * fill the array of output
   *
   */

  (*outputArray).search         = burst;
  (*outputArray).significant    = 1;
  (*outputArray).states         = NULL;
  (*outputArray).optional       = NULL;
  (*outputArray).results        = NULL;

  /* May not be correct?  Does this need to be non-zero */
  (*outputArray).indexNumber = 0;


  /* 
   *
   * alloc memory for the database linked list
   *
   */

  { 
    INT4 numEvents = params->numEvents;

    /* entry: start_time */
    dbEntry = (*outputArray).results = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = NULL;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "start_time" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( numEvents*sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.int4s)[i] = event->startTime;
    }

    prev = dbEntry; 

    /* entry: start_time_ns */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "start_time_ns" );
    dbEntry->type       = int_4s;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( numEvents*sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.int4s)[i] = event->startTimeNS;
    }

    prev = dbEntry;

    /* entry: duration */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "duration" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->duration;
    }

    prev = dbEntry;

    
    /* entry: central frequency */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "central_freq" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->centralFrequency;
    }

    prev = dbEntry; 

    /* entry: bandwidth */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "bandwidth" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->bandwidth;
    }
    prev = dbEntry; 


    /* entry: amplitude must be non-null, but needs fixing */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "amplitude" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->amplitude;
    }

    prev = dbEntry;

    /* entry: excess power */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "snr" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      (dbEntry->rows.real4)[i] = event->excessPower;
    }
    prev = dbEntry; 

    /* entry: confidence */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev; 
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "confidence" );
    dbEntry->type       = real_4;
    dbEntry->numberRows = numEvents;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( numEvents*sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
        (dbEntry->rows.real4)[i] = event->confidence;
    }

    prev = dbEntry;

    /* entry: ifo */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "ifo" );
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = numEvents;
    stringLength =  APPLYSEARCHH_STRLENGTH;
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( dbEntry->numberRows * sizeof(UINT4) );
    dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      *(dbEntry->rowDimensions + i) = stringLength;
      strncpy( dbEntry->rows.chars + i*stringLength, burstEventList->ifo, 
          stringLength*sizeof(CHAR) );
    }

    prev = dbEntry; 
    
    /* entry: search */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "search" );
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = numEvents;
    stringLength =  strlen( APPLYSEARCHH_SEARCHNAME );
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( dbEntry->numberRows * sizeof(UINT4) );
    dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
    for ( i = 0, event = burstEvent; (i < params->numEvents) && event; 
        ++i, event = event->nextEvent )
    {
      *(dbEntry->rowDimensions + i) = stringLength;
      strncpy( dbEntry->rows.chars + i*stringLength, 
          APPLYSEARCHH_SEARCHNAME, stringLength*sizeof(CHAR) );
    }

    dbEntry->next = NULL;

  }

  RETURN (status);
}
