/**** <lalVerbatim file="StochasticConditionDataHV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticConditionData.h,v 1.11 2002/03/04 21:35:51 whelan Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticConditionDataH">
\section{Header \texttt{StochasticConditionData.h}}
\label{s:StochasticConditionData.h}

Header file for a routine that does one-time processing in preparation 
for the stochastic search.

\subsection*{Synopsis}
\begin{verbatim}
#include "StochasticConditionData.h"
\end{verbatim}

\subsection*{Error conditions}
\input{StochasticConditionDataHErrTab}

\subsection*{Structures}

None.

*************************************************** </lalLaTeX> */

#ifndef _STOCHASTICCONDITIONDATA_H
#define _STOCHASTICCONDITIONDATA_H

#include "LALWrapperInterface.h"
#include "Stochastic.h"

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (STOCHASTICCONDITIONDATAH, "$Id: StochasticConditionData.h,v 1.11 2002/03/04 21:35:51 whelan Exp $");

/******** <lalErrTable file="StochasticConditionDataHErrTab"> ********/
#define STOCHASTICCONDITIONDATAH_ENULL      1
#define STOCHASTICCONDITIONDATAH_EMSTR      2
#define STOCHASTICCONDITIONDATAH_ENSEQ      3
#define STOCHASTICCONDITIONDATAH_ETYPE      4 
#define STOCHASTICCONDITIONDATAH_EDIMS      5
#define STOCHASTICCONDITIONDATAH_ESIZE      6
#define STOCHASTICCONDITIONDATAH_ENAME      7
#define STOCHASTICCONDITIONDATAH_EALOC      8
#define STOCHASTICCONDITIONDATAH_EUNIT      9
#define STOCHASTICCONDITIONDATAH_ESTID     10

#define STOCHASTICCONDITIONDATAH_MSGENULL   "Null pointer"
#define STOCHASTICCONDITIONDATAH_MSGEMSTR   "Not search master"
#define STOCHASTICCONDITIONDATAH_MSGENSEQ   "Wrong number of sequences in InPut structure"
#define STOCHASTICCONDITIONDATAH_MSGETYPE   "Input data has wrong type"
#define STOCHASTICCONDITIONDATAH_MSGEDIMS   "Input data has wrong dimensions"
#define STOCHASTICCONDITIONDATAH_MSGESIZE   "Input data has wrong size"
#define STOCHASTICCONDITIONDATAH_MSGENAME   "Wrong data name in InPut structure"
#define STOCHASTICCONDITIONDATAH_MSGEALOC   "Memory allocation error"
#define STOCHASTICCONDITIONDATAH_MSGEUNIT   "Input data had unexpected units"
#define STOCHASTICCONDITIONDATAH_MSGESTID   "Detector site identifier unrecognized"
/******** </lalErrTable> ********/

/* the comments by which the incoming channels are identified */
#define STOCHASTICCONDITIONDATAH_DATA1COMMENT "data1"
#define STOCHASTICCONDITIONDATAH_DATA2COMMENT "data2"
#define STOCHASTICCONDITIONDATAH_SPEC1COMMENT "spectrum1"
#define STOCHASTICCONDITIONDATAH_SPEC2COMMENT "spectrum2"
#define STOCHASTICCONDITIONDATAH_RESP1COMMENT "response1"
#define STOCHASTICCONDITIONDATAH_RESP2COMMENT "response2"

#ifdef  __cplusplus
}
#endif /* C++ protection */

#endif /* _STOCHASTICCONDITIONDATA_H */

/******************************* <lalLaTeX file="StochasticConditionDataH">
  
\vfill{\footnotesize\input{StochasticConditionDataHV}}

\newpage\input{StochasticConditionDataC}

*************************************************** </lalLaTeX> */
