/*************** <lalVerbatim file="InitCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Init.c,v 1.7 2003/05/27 17:40:24 igor Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <lal/LALWavelet.h>
#include <Wave.h>
#include <BuildDB.h>
#include <float.h>

static void _defaults(WaveParams *params);


NRCSID(INITC, "$Id" );

static void _defaults(WaveParams *params)
{
  params->nonZeroFraction=0.3;
  params->usePixelMixer=FALSE;
  params->seed=time(NULL);
  params->usePixelSwapOne=FALSE;
  params->usePixelSwapTwo=FALSE;
  params->timeWindowPixels=0;
  params->freqWindowPixels=0;
  params->coincidenceLevel=CROSS_CO;
  params->minAmp4SinglePixels=0.0;
  params->minAmp4ClusterExtension=0.0;
  params->aura=FALSE;
  params->minClusterSize=3;
  params->maxClusterSize=10000;
  params->minLikelihood=0.0;
  params->maxLikelihood=FLT_MAX;
  params->minCorrelation=-FLT_MAX;
  params->maxCorrelation=FLT_MAX;
  params->minPower=0.0;
  params->maxPower=FLT_MAX;
  params->minEnergy=0.0;
  params->maxEnergy=FLT_MAX;
  params->simulationType=0;
  params->ncalls=0;
  bzero(params->nameOne,MAX_NAME_LENGTH);
  bzero(params->nameTwo,MAX_NAME_LENGTH);
  strcpy(params->nameOne,"default one");
  strcpy(params->nameTwo,"default two");
  bzero(params->comment,MAX_NAME_LENGTH);
  params->start.gpsSeconds=0;
  params->start.gpsNanoSeconds=0;
  params->stop.gpsSeconds=0;
  params->stop.gpsNanoSeconds=0;
  params->oneIn->type=params->twoIn->type=BIORTHOGONAL;
  params->oneIn->treeType=params->twoIn->treeType=BINARY;
  params->oneIn->level=params->twoIn->level=5;
  params->oneIn->border=params->twoIn->border=B_POLYNOM;
  params->oneIn->HPFilterLength=params->twoIn->HPFilterLength=10;
  params->oneIn->LPFilterLength=params->twoIn->LPFilterLength=10;
  params->o1=FALSE;
  params->o2=FALSE;
  params->p1=FALSE;
  params->p2=FALSE;
  params->co1=FALSE;
  params->co2=FALSE;
  params->cl1=TRUE;
  params->cl2=TRUE;
  params->blobP=FALSE;
  params->blobO=FALSE;
  strcpy(params->outputToFile,"PT");
  params->reuseCluster=FALSE;
  params->strain=0.0;
  params->timeShift=0.0;
  params->noise_rms_flag=FALSE;
  params->calibration_max_freq=2048.0;
}



void LALInitSearch(LALStatus *status,
		   void **searchParams,
		   LALInitSearchParams *initSearchParams
		   )
{ 
  INT4 ac;
  CHAR **av;
  INT4 i;
  WaveParams *params;
  int pair;

  INITSTATUS( status, "LALInitSearch", INITC );
  ATTATCHSTATUSPTR( status );

  /*Memory allocation*/

  params=(WaveParams *)LALMalloc(sizeof(WaveParams));

  LALAllocateWavelet(status->statusPtr, &params->oneIn);
  CHECKSTATUSPTR(status);
  LALAllocateWavelet(status->statusPtr, &params->twoIn);
  CHECKSTATUSPTR(status);

  _defaults(params);

  params->oneOutPT=
    (OutputPercentileWavelet*)LALCalloc(1,sizeof(OutputPercentileWavelet));
  params->twoOutPT=
    (OutputPercentileWavelet*)LALCalloc(1,sizeof(OutputPercentileWavelet));
  params->oneOutPS=(OutputPixelSwapWavelet*)LALCalloc(1,sizeof(OutputPixelSwapWavelet));
  params->twoOutPS=(OutputPixelSwapWavelet*)LALCalloc(1,sizeof(OutputPixelSwapWavelet));
  params->oneOutPM=(OutputPixelMixerWavelet*)LALCalloc(1,sizeof(OutputPixelMixerWavelet));
  params->twoOutPM=(OutputPixelMixerWavelet*)LALCalloc(1,sizeof(OutputPixelMixerWavelet));
  params->coincidenceOut=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->coincidenceOutPS=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->coincidenceOutPM=(OutputCoincidenceWavelet *)LALCalloc(1,sizeof(OutputCoincidenceWavelet));
  params->oneOutCluster=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->oneOutClusterPS=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->oneOutClusterPM=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutCluster=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutClusterPS=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));
  params->twoOutClusterPM=(OutputClusterWavelet *)LALCalloc(1,sizeof(OutputClusterWavelet));

  *searchParams=params;

  /*Default parameters are overwritten if specified in -filterparams */

  ac=initSearchParams->argc;
  av=initSearchParams->argv;

  for(i=1;i<ac;i++)
    {
      pair=0;
      if(strcmp(av[i],"-WaveletType")==0)
	{
	  params->oneIn->type=params->twoIn->type=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletTreeType")==0)
	{
	  params->oneIn->treeType=params->twoIn->treeType=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletLevel")==0)
	{
	  params->oneIn->level=
	    params->twoIn->level=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletBorder")==0)
	{
	  params->oneIn->border=params->twoIn->border=
	    (INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletHPFilterLength")==0)
	{
	  params->oneIn->HPFilterLength=
	    params->twoIn->HPFilterLength=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-WaveletLPFilterLength")==0)
	{
	  params->oneIn->LPFilterLength=
	    params->twoIn->LPFilterLength=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-ch1")==0)
	{
	  strncpy(params->nameOne,av[i+1],MAX_NAME_LENGTH-1);
	  pair++;
	}
      if(strcmp(av[i],"-ch2")==0)
	{
	  strncpy(params->nameTwo,av[i+1],MAX_NAME_LENGTH-1);
	  pair++;
	}
      if(strcmp(av[i],"-comment")==0)
	{
	  strncpy(params->comment,av[i+1],MAX_NAME_LENGTH-1);
	  pair++;
	}

      if(strcmp(av[i],"-nonZeroFraction")==0)
	{
	  params->nonZeroFraction=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-seed")==0)
	{
	  params->seed=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-timeWindowPixels")==0)
	{
	  params->timeWindowPixels=(INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-freqWindowPixels")==0)
	{
	  params->freqWindowPixels=(INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-coincidenceLevel")==0)
	{
	  params->coincidenceLevel=(INT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minAmp4SinglePixels")==0)
	{
	  params->minAmp4SinglePixels=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minAmp4ClusterExtension")==0)
	{
	  params->minAmp4ClusterExtension=(REAL4)atof(av[i+1]);
          pair++;
	}
      if(strcmp(av[i],"-simulationType")==0)
	{
	  params->simulationType=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-halo")==0)
	{
	  params->aura=(BOOLEAN)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minLikelihood")==0)
	{
	  params->minLikelihood=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxLikelihood")==0)
	{
	  params->maxLikelihood=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minPower")==0)
	{
	  params->minPower=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxPower")==0)
	{
	  params->maxPower=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minEnergy")==0)
	{
	  params->minEnergy=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxEnergy")==0)
	{
	  params->maxEnergy=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minCorrelation")==0)
	{
	  params->minCorrelation=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxCorrelation")==0)
	{
	  params->maxCorrelation=(REAL4)atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-minClusterSize")==0)
	{
	  params->minClusterSize=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-maxClusterSize")==0)
	{
	  params->maxClusterSize=(UINT4)atoi(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-pixelMixer")==0)
	{
	  params->usePixelMixer=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-pixelSwapOne")==0)
	{
	  params->usePixelSwapOne=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-pixelSwapTwo")==0)
	{
	  params->usePixelSwapTwo=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-o1")==0)
	{
	  params->o1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-o2")==0)
	{
	  params->o2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-p1")==0)
	{
	  params->p1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-p2")==0)
	{
	  params->p2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-co1")==0)
	{
	  params->co1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-co2")==0)
	{
	  params->co2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-cl1")==0)
	{
	  params->cl1=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-cl2")==0)
	{
	  params->cl2=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE; 
	  pair++;
	}
      if(strcmp(av[i],"-blobP")==0)
	{
	  params->blobP=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-blobO")==0)
	{
	  params->blobO=
	    strcmp(av[i+1],"0")==0 ? FALSE : TRUE;
	  pair++;
	}
      if(strcmp(av[i],"-outputToFile")==0)
	{
	  strcpy(params->outputToFile,av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-reuseCluster")==0)
	{
	  params->reuseCluster=((strncmp(av[i+1],"1",1)==0) ? TRUE:FALSE);
	  pair++;
	}
      if(strcmp(av[i],"-noise_rms_flag")==0)
	{
	  params->noise_rms_flag=((strncmp(av[i+1],"1",1)==0) ? TRUE : FALSE);
	  pair++;
	}
      if(strcmp(av[i],"-calibration_max_freq")==0)
	{
	  params->calibration_max_freq=atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-strain")==0)
	{
	  params->strain=atof(av[i+1]);
	  pair++;
	}
      if(strcmp(av[i],"-timeShift")==0)
	{
	  params->timeShift=atof(av[i+1]);
	  pair++;
	}
    }

  if(params->noise_rms_flag)
    {
      params->onePSD=(REAL4FrequencySeries*)LALCalloc(1,sizeof(REAL4FrequencySeries));
      params->onePSD->data=(REAL4Sequence*)LALCalloc(1,sizeof(REAL4Sequence));
      params->twoPSD=(REAL4FrequencySeries*)LALCalloc(1,sizeof(REAL4FrequencySeries));
      params->twoPSD->data=(REAL4Sequence*)LALCalloc(1,sizeof(REAL4Sequence));
    }
  else
    {
      params->onePSD=NULL;
      params->twoPSD=NULL;
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

