/*************** <lalVerbatim file="OutputCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Output.c,v 1.9 2003/05/27 17:40:24 igor Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <Wave.h>
#include <BuildDB.h>

NRCSID(OUTPUTC, "$Id" );

extern const char *lalCVSTag;
static const char *lalWrapperCVSTag = LAL_WRAPPER_CVS_TAG;


void LALBuildOutputWave(LALStatus *status,
			WaveParams *my, 
			LALSearchOutput *output
			)
{  
  int counter=0;
  LALSummaryDBWaveInput summary;
  LALClustersDBWaveInput db;
  LALBlobsDBWaveInput b;
  LALClustersTSWaveInput ts;
  LALClustersParametersWaveInput params;
  LALClustersMediansWaveInput medians;
  LALClustersVolumesWaveInput volumes;
  LALClustersListWaveInput cList;
  LALClustersPMaskWaveInput pMask;
  
  INITSTATUS( status, "LALBuildOutputWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  output->numOutput=23;

  if(my->o1)  output->numOutput++;
  if(my->o2)  output->numOutput++;
  if(my->p1)  output->numOutput+=2;
  if(my->p2)  output->numOutput+=2;
  if(my->co1) output->numOutput++;
  if(my->co2) output->numOutput++;
  if(my->cl1) output->numOutput+=11;
  if(my->cl2) output->numOutput+=11;
  if(my->blobP || my->blobO) output->numOutput++;

  output->result=(outPut*)LALCalloc(output->numOutput,sizeof(outPut));

  LALSelectWave(status->statusPtr,my);
  CHECKSTATUSPTR(status); 

  summary.my=my;
  summary.counter=&counter;
  summary.comment[0]='\0';
  strcat(summary.comment,my->oneOutCluster->w->wavelet->data->name);
  strcat(summary.comment,";");
  strcat(summary.comment,my->twoOutCluster->w->wavelet->data->name);
  strcat(summary.comment,";one:orig;two:orig;");
  summary.nevents=my->oneOutCluster->w->clusterCountFinal + 
    my->twoOutCluster->w->clusterCountFinal;
  summary.nnodes=2;
  summary.strain=my->strain;
  summary.timeShift=my->timeShift;
  summary.nevents_gw_1=my->oneOutCluster->w->clusterCountFinal;
  summary.nevents_gw_2=my->twoOutCluster->w->clusterCountFinal;


  LALSetAmplitudesWavelet(status->statusPtr,my->oneOutCluster->w);
  CHECKSTATUSPTR(status); 
  LALSetAmplitudesWavelet(status->statusPtr,my->twoOutCluster->w);
  CHECKSTATUSPTR(status); 

  if(my->oneOutCluster->w->clusterCountFinal!=0)
    {
      my->oneOutCluster->w->clusterType=ORIGINAL_CL;
      db.my=my;
      db.w=my->oneOutCluster->w;
      db.theOther=my->twoOutCluster->w;
      db.counter=&counter;
      db.comment[0]='\0';
      db.startEventID=0;
      strcat(db.comment,";one;");
      LALClustersDBWave(status->statusPtr, &db, output);
      CHECKSTATUSPTR(status);
      
      b.my=my;
      b.w=my->oneOutCluster->w;
      b.theOther=my->twoOutCluster->w;
      b.counter=&counter;
      b.comment[0]='\0';
      b.startEventID=0;
      strcpy(b.comment,"one");
      LALBlobsDBWave(status->statusPtr, &b, output);
      CHECKSTATUSPTR(status);
    }

  if(my->twoOutCluster->w->clusterCountFinal!=0)
    {
      my->twoOutCluster->w->clusterType=ORIGINAL_CL;
      db.my=my;
      db.w=my->twoOutCluster->w;
      db.theOther=my->oneOutCluster->w;
      db.counter=&counter;
      db.comment[0]='\0';
      db.startEventID+=my->oneOutCluster->w->clusterCountFinal;
      strcat(db.comment,";two;");
      LALClustersDBWave(status->statusPtr, &db, output);
      CHECKSTATUSPTR(status);
      
      b.my=my;
      b.w=my->twoOutCluster->w;
      b.theOther=my->oneOutCluster->w;
      b.counter=&counter;
      b.comment[0]='\0';
      b.startEventID=my->oneOutCluster->w->clusterCountFinal;
      strcpy(b.comment,"two");
      LALBlobsDBWave(status->statusPtr, &b, output);
      CHECKSTATUSPTR(status);
    }

  params.my=my;
  params.counter=&counter;
  params.w1=my->oneOutCluster->w;
  params.w2=my->twoOutCluster->w;
  params.comment[0]='\0';
  strcpy(params.comment,
	 "pMaskCount,clusterCount, all nonZeroFractions for two channels");
  LALClustersParametersWave(status->statusPtr,&params, output);
  CHECKSTATUSPTR(status);

  if(my->usePixelSwapOne || my->usePixelSwapTwo)
    {
      if(my->usePixelSwapOne) strcat(summary.comment,"PS1;");
      if(my->usePixelSwapTwo) strcat(summary.comment,"PS2;");
      summary.nevents+=my->oneOutClusterPS->w->clusterCountFinal + 
	my->twoOutClusterPS->w->clusterCountFinal;
      summary.nevents_swp_1=my->oneOutClusterPS->w->clusterCountFinal;
      summary.nevents_swp_2=my->twoOutClusterPS->w->clusterCountFinal;


      if(my->oneOutClusterPS->w!=NULL && my->twoOutClusterPS->w!=NULL)
	{
	  if(my->oneOutClusterPS->w->clusterCountFinal!=0)
	    {
	      my->oneOutClusterPS->w->clusterType=SWAPPED_CL;
	      db.my=my;
	      db.w=my->oneOutClusterPS->w;
	      db.theOther=my->twoOutClusterPS->w;
	      db.counter=&counter;
	      db.comment[0]='\0';
	      strcat(db.comment,";one;");
	      if(my->usePixelSwapOne) strcat(db.comment,"PS1;");
	      if(my->usePixelSwapTwo) strcat(db.comment,"PS2;");
	      LALClustersDBWave(status->statusPtr, &db, output);
	      CHECKSTATUSPTR(status);
	    }

	  if(my->twoOutClusterPS->w->clusterCountFinal!=0)
	    {
	      my->twoOutClusterPS->w->clusterType=SWAPPED_CL;
	      db.my=my;
	      db.w=my->twoOutClusterPS->w;
	      db.theOther=my->oneOutClusterPS->w;
	      db.counter=&counter;
	      db.comment[0]='\0';
	      strcat(db.comment,";two;");
	      if(my->usePixelSwapOne) strcat(db.comment,"PS1;");
	      if(my->usePixelSwapTwo) strcat(db.comment,"PS2;");
	      LALClustersDBWave(status->statusPtr, &db, output);
	      CHECKSTATUSPTR(status);
	    }

	  params.my=my;
	  params.counter=&counter;
	  params.w1=my->oneOutClusterPS->w;
	  params.w2=my->twoOutClusterPS->w;
	  params.comment[0]='\0';
	  strcpy(params.comment,
		 "pMaskCount,clusterCount, all nonZeroFractions for two channels");
	  if(my->usePixelSwapOne) strcat(params.comment,":PS1");
	  if(my->usePixelSwapTwo) strcat(params.comment,":PS2:");
	  LALClustersParametersWave(status->statusPtr,&params, output);
	  CHECKSTATUSPTR(status);
	}
    }

  if(my->usePixelMixer)
    {

      strcat(summary.comment,"PM;");
      summary.nevents+=my->oneOutClusterPM->w->clusterCountFinal +
	my->twoOutClusterPM->w->clusterCountFinal;
      summary.nevents_pm_1=my->oneOutClusterPM->w->clusterCountFinal;
      summary.nevents_pm_2=my->twoOutClusterPM->w->clusterCountFinal;

      if(my->oneOutClusterPM->w!=NULL && my->twoOutClusterPM->w!=NULL)
	{
	  if(my->oneOutClusterPM->w->clusterCountFinal!=0)
	    {
	      my->oneOutClusterPM->w->clusterType=MIXED_CL;
	      db.my=my;
	      db.w=my->oneOutClusterPM->w;
	      db.theOther=my->twoOutClusterPM->w;
	      db.counter=&counter;
	      db.comment[0]='\0';
	      strcat(db.comment,";one;PM;");
	      LALClustersDBWave(status->statusPtr, &db, output);
	      CHECKSTATUSPTR(status);
	    }

	  if(my->twoOutClusterPM->w->clusterCountFinal!=0)
	    {
	      my->twoOutClusterPM->w->clusterType=MIXED_CL;
	      db.my=my;
	      db.w=my->twoOutClusterPM->w;
	      db.theOther=my->oneOutClusterPM->w;
	      db.counter=&counter;
	      db.comment[0]='\0';
	      strcat(db.comment,";two;PM;");
	      LALClustersDBWave(status->statusPtr, &db, output);
	      CHECKSTATUSPTR(status);
	    }
      
	  params.my=my;
	  params.counter=&counter;
	  params.w1=my->oneOutClusterPM->w;
	  params.w2=my->twoOutClusterPM->w;
	  params.comment[0]='\0';
	  strcpy(params.comment,
		 "pMaskCount,clusterCount, all nonZeroFractions for two channels:PM");
	  LALClustersParametersWave(status->statusPtr,&params, output);
	  CHECKSTATUSPTR(status);
	}
    }

  summary.counter=&counter;
  LALSummaryDBWave(status->statusPtr, &summary, output);
  CHECKSTATUSPTR(status);

  if(my->o1)
    {
      ts.my=my;
      LALAssignREAL4TimeSeries(status->statusPtr,&ts.ts,my->oneIn->data);
      CHECKSTATUSPTR(status);
      ts.counter=&counter;
      strcpy(ts.comment,"original time series one");
      LALClustersTSWave(status->statusPtr, &ts, output);
      CHECKSTATUSPTR(status);
    }
  if(my->o2)
    {
      ts.my=my;
      LALAssignREAL4TimeSeries(status->statusPtr,&ts.ts,my->twoIn->data);
      CHECKSTATUSPTR(status);
      ts.counter=&counter;
      strcpy(ts.comment,"original time series two");
      LALClustersTSWave(status->statusPtr, &ts, output);
      CHECKSTATUSPTR(status);
    }
  if(my->p1)
    {
      medians.my=my;
      medians.w=my->oneOutPT->out;
      medians.counter=&counter;
      strcpy(medians.comment,"medians one");
      LALClustersMediansWave(status->statusPtr, &medians, output);
      CHECKSTATUSPTR(status);

      if(strcmp(my->outputToFile,"PT")==0)
	{
	  ts.my=my;
	  ts.ts=my->oneOutPT->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after percentile transform");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
	  
      if(strcmp(my->outputToFile,"PS")==0 && my->usePixelSwapOne)
	{
	  ts.my=my;
	  ts.ts=my->oneOutPS->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after pixel swap");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->oneOutPM->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after pixel mixing");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
    }

  if(my->p2)
    {
      medians.my=my;
      medians.w=my->twoOutPT->out;
      medians.counter=&counter;
      strcpy(medians.comment,"medians two");
      LALClustersMediansWave(status->statusPtr, &medians, output);
      CHECKSTATUSPTR(status);

      if(strcmp(my->outputToFile,"PT")==0)
	{      
	  ts.my=my;
	  ts.ts=my->twoOutPT->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after percentile transform");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

      if(strcmp(my->outputToFile,"PS")==0 && my->usePixelSwapTwo)
	{
	  ts.my=my;
	  ts.ts=my->twoOutPS->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after pixel swap");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->twoOutPM->out->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after pixel mixing");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
    }

  if(my->co1)
    {
      if(strcmp(my->outputToFile,"PT")==0)
	{      
	  ts.my=my;
	  ts.ts=my->coincidenceOut->one->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after coincidence");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

      if(strcmp(my->outputToFile,"PS")==0 && (my->usePixelSwapOne || my->usePixelSwapTwo))
	{
	  ts.my=my;
	  ts.ts=my->coincidenceOutPS->one->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after coincidence and pixel swapping");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->coincidenceOut->one->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after coincidence and pixel mixing");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
    }
  if(my->co2)
    {
      if(strcmp(my->outputToFile,"PT")==0)
	{      
	  ts.my=my;
	  ts.ts=my->coincidenceOut->two->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after coincidence");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

      if(strcmp(my->outputToFile,"PS")==0 && (my->usePixelSwapOne || my->usePixelSwapTwo))
	{
	  ts.my=my;
	  ts.ts=my->coincidenceOutPS->two->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after coincidence and pixel swapping");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}
      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->coincidenceOutPM->two->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after coincidence and pixel mixing");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	}

    }

      
  if(my->cl1)
    {
      if(strcmp(my->outputToFile,"PT")==0)
	{      
	  ts.my=my;
	  ts.ts=my->oneOutCluster->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after clustering");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);

	  volumes.my=my;
	  volumes.w=my->oneOutCluster->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes one");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);
	  
	  cList.my=my;
	  cList.w=my->oneOutCluster->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList one");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);
	  
	  pMask.my=my;
	  pMask.w=my->oneOutCluster->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask one");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status); 
	}     

      if(strcmp(my->outputToFile,"PS")==0 && (my->usePixelSwapOne || my->usePixelSwapTwo))
	{
	  ts.my=my;
	  ts.ts=my->oneOutClusterPS->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after clustering and pixel swapping");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);

	  volumes.my=my;
	  volumes.w=my->oneOutClusterPS->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes one after pixel swapping");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);

	  cList.my=my;
	  cList.w=my->oneOutClusterPS->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList one after pixel swapping");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);

	  pMask.my=my;
	  pMask.w=my->oneOutClusterPS->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask one after pixel swapping");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status);      
	}
      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->oneOutClusterPM->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series one after clustering and pixel mixing");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);

	  volumes.my=my;
	  volumes.w=my->oneOutClusterPM->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes one after pixel mixing");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);

	  cList.my=my;
	  cList.w=my->oneOutClusterPM->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList one after pixel mixing");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);

	  pMask.my=my;
	  pMask.w=my->oneOutClusterPM->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask one after pixel mixing");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status);      
	}
    }
  if(my->cl2)
    {
      if(strcmp(my->outputToFile,"PT")==0)
	{      
	  ts.my=my;
	  ts.ts=my->twoOutCluster->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after clustering");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);

	  volumes.my=my;
	  volumes.w=my->twoOutCluster->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes two");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);
	  
	  cList.my=my;
	  cList.w=my->twoOutCluster->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList two");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);
	  
	  pMask.my=my;
	  pMask.w=my->twoOutCluster->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask two");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status);      
	}

      if(strcmp(my->outputToFile,"PS")==0 && (my->usePixelSwapOne || my->usePixelSwapTwo))
	{
	  ts.my=my;
	  ts.ts=my->twoOutClusterPS->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after clustering");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);

	  volumes.my=my;
	  volumes.w=my->twoOutClusterPS->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes two after pixel swapping");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);
	  
	  cList.my=my;
	  cList.w=my->twoOutClusterPS->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList two after pixel swapping");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);
	  
	  pMask.my=my;
	  pMask.w=my->twoOutClusterPS->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask two after pixel swapping");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status);      
	}

      if(strcmp(my->outputToFile,"PM")==0 && my->usePixelMixer)
	{
	  ts.my=my;
	  ts.ts=my->twoOutClusterPM->w->wavelet->data;
	  ts.counter=&counter;
	  strcpy(ts.comment,"time series two after clustering PM");
	  LALClustersTSWave(status->statusPtr, &ts, output);
	  CHECKSTATUSPTR(status);
	  
	  volumes.my=my;
	  volumes.w=my->twoOutClusterPM->w;
	  volumes.counter=&counter;
	  strcpy(volumes.comment,"volumes two after PM");
	  LALClustersVolumesWave(status->statusPtr,&volumes,output);
	  CHECKSTATUSPTR(status);
	  
	  cList.my=my;
	  cList.w=my->twoOutClusterPM->w;
	  cList.counter=&counter;
	  strcpy(cList.comment,"cList two after PM");
	  LALClustersListWave(status->statusPtr,&cList,output);
	  CHECKSTATUSPTR(status);
	  
	  pMask.my=my;
	  pMask.w=my->twoOutClusterPM->w;
	  pMask.counter=&counter;
	  strcpy(pMask.comment,"pMask two after pixel PM");
	  LALClustersPMaskWave(status->statusPtr,&pMask,output);
	  CHECKSTATUSPTR(status);      
	}
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void LALSummaryDBWave(LALStatus *status,
		      LALSummaryDBWaveInput *input,
		      LALSearchOutput *output
		      )
{
  int ix;
  LALBLOB blob;
  dataBase *dbase=NULL;

  INITSTATUS( status, "LALSummaryDBWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","comment");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;
  blob.data=input->my->comment;
  blob.size=strlen(input->my->comment);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","lalwrapper_cvs_tag");
  CHECKSTATUSPTR(status);
  blob.data=(void*)lalWrapperCVSTag;
  blob.size=strlen(lalWrapperCVSTag);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","lal_cvs_tag");
  CHECKSTATUSPTR(status);
  blob.data=(void*)lalCVSTag;
  blob.size=strlen(lalCVSTag);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","shared_object");
  CHECKSTATUSPTR(status);
  blob.data="waveburst";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","in_start_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->start.gpsSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","in_start_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->start.gpsNanoSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","out_start_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->start.gpsSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","out_start_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->start.gpsNanoSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","in_end_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->stop.gpsSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","in_end_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->stop.gpsNanoSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","out_end_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->stop.gpsSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","out_end_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->my->stop.gpsNanoSeconds;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","nevents");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=input->nevents;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summary","nnodes");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr,dbase,int_4s,1);
  dbase->rows.int4s[0]=2;

/*    LALCreateNextDBEntry(status->statusPtr, &dbase,  */
/*  		       "search_summary","process_id"); */
/*    CHECKSTATUSPTR(status); */
/*    blob.data="processgroup:process:process_id:0"; */
/*    blob.size=strlen(blob.data); */
/*    LALCreateDBEntryBLOB(status->statusPtr, dbase,  */
/*  		       &blob, 1); */
/*    CHECKSTATUSPTR(status); */

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="strain";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=input->my->strain;
  CHECKSTATUSPTR(status);

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="timeshift";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
		       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=input->my->timeShift;
  CHECKSTATUSPTR(status);

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_gw_1";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_gw_1;
  CHECKSTATUSPTR(status);

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_gw_2";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_gw_2;
  CHECKSTATUSPTR(status);

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_swp_1";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_swp_1;
  CHECKSTATUSPTR(status);


  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_swp_2";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_swp_2;
  CHECKSTATUSPTR(status);


  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_pm_1";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_pm_1;
  CHECKSTATUSPTR(status);


  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;

  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","name");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;

  blob.data="nevents_pm_2";
  blob.size=strlen(blob.data);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, &blob, 1);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr,&dbase,
                       "search_summvars","value");
  CHECKSTATUSPTR(status);

  LALCreateDBEntryData(status->statusPtr,dbase,real_8,1);
  dbase->rows.real8[0]=(REAL8)input->nevents_pm_2;
  CHECKSTATUSPTR(status);


  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALClustersDBWave(LALStatus *status,
		       LALClustersDBWaveInput *input,
		       LALSearchOutput *output
		       )
{
  int ix;

  LALBLOB blob;
  dataBase *dbase=NULL;
  CHAR tmp[LALNameLength];
  CHAR ifo[3];
  CHAR *ptr;
  int i,j;

  INITSTATUS( status, "LALClustersDBWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;
  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","comment");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;
  blob.data=input->comment;
  blob.size=strlen(input->comment);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","ifo_1");
  CHECKSTATUSPTR(status);
  strcpy(tmp,input->w->wavelet->data->name);
  ptr=strtok(tmp,":");
  strcpy(ifo,ptr);
  blob.data=ptr;
  blob.size=2;
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","ifo_2");
  CHECKSTATUSPTR(status);
  strcpy(tmp,input->theOther->wavelet->data->name);
  ptr=strtok(tmp,":");
  strcpy(ifo,ptr);
  blob.data=ptr;
  blob.size=2;
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","channel_1");
  CHECKSTATUSPTR(status);
  blob.data=input->w->wavelet->data->name;
  blob.size=strlen(input->w->wavelet->data->name);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","channel_2");
  CHECKSTATUSPTR(status);
  blob.data=input->theOther->wavelet->data->name;
  blob.size=strlen(input->theOther->wavelet->data->name);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);


  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","start_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{
	  dbase->rows.int4s[j++]=
	    input->w->absoluteStartTime[i].gpsSeconds;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","start_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{
	  dbase->rows.int4s[j++]=
	    input->w->absoluteStartTime[i].gpsNanoSeconds;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","stop_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{
	  dbase->rows.int4s[j++]=
	    input->w->absoluteStopTime[i].gpsSeconds;
	}
    }


  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","stop_time_ns");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{
	  dbase->rows.int4s[j++]=
	    input->w->absoluteStopTime[i].gpsNanoSeconds;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","duration");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_8, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{     
	  dbase->rows.real8[j++]=input->w->duration[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","rel_start_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_8, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real8[j++]=input->w->relativeStartTime[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","rel_stop_time");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_8, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real8[j++]=input->w->relativeStopTime[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","power");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->power[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","xcorrelation");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->correlation[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","likelihood");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->likelihood[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase,
                       "waveburst","noise_rms");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase,
                       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
        {
	  if(input->w->noise_rms_flag)
	    dbase->rows.real4[j++]=input->w->noise_rms[i];
	  else
	    dbase->rows.real4[j++]=-1.29;
        }
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase,
                       "waveburst","extra");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase,
                       real_8, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
        {
          dbase->rows.real8[j++]=-2.3;
        }
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","max_amplitude");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->maxAmplitude[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","start_frequency");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->startFrequency[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","stop_frequency");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->stopFrequency[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","bandwidth");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       real_4, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.real4[j++]=input->w->bandwidth[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","volume");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
/*   dbase->rows.int4s=(INT4*)LALCalloc(input->w->clusterCount,sizeof(INT4)); */
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.int4s[j++]=input->w->volumes[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","core_size");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.int4s[j++]=input->w->coreSize[i];
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","cluster_id");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.int4s[j]=j;
	  j++;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","sim_type");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.int4s[j++]=input->my->simulationType;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst","cluster_type");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  dbase->rows.int4s[j++]=input->w->clusterType;
	}
    }

  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{           
	  LALCreateNextDBEntry(status->statusPtr, &dbase, 
			       "waveburst","event_id");
	  CHECKSTATUSPTR(status);
	  sprintf(tmp,"waveburstgroup:event_id:%d",(j + input->startEventID));j++;
	  blob.data=tmp;
	  blob.size=strlen(tmp);
	  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
			       &blob, 1);
	  CHECKSTATUSPTR(status);
	}
    }

/*    LALCreateNextDBEntry(status->statusPtr, &dbase,  */
/*  		       "waveburst","process_id"); */
/*    CHECKSTATUSPTR(status); */
/*    blob.data="processgroup:process:process_id:0"; */
/*    blob.size=strlen(blob.data); */
/*    LALCreateDBEntryBLOB(status->statusPtr, dbase,  */
/*  		       &blob, input->w->clusterCount); */
/*    CHECKSTATUSPTR(status); */


  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALClustersTSWave(LALStatus *status,
		       LALClustersTSWaveInput *input,
		       LALSearchOutput *output
		       )
{
  int ix;
  INITSTATUS( status, "LALClustersTSWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->ts->name);
  strcpy(output->result[ix].optional->units,"ADC");
  strcpy(output->result[ix].optional->comment,input->comment);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=real_4;

  output->result[ix].optional->range.dTime.startSec=
    input->my->start.gpsSeconds;
  output->result[ix].optional->range.dTime.startNan=
    input->my->start.gpsNanoSeconds;
  output->result[ix].optional->range.dTime.stopSec=
    input->my->stop.gpsSeconds;
  output->result[ix].optional->range.dTime.stopNan=
    input->my->stop.gpsNanoSeconds;
  output->result[ix].optional->range.dTime.numberSamples=
    input->ts->data->length;
  output->result[ix].optional->range.dTime.timeStepSize=
    input->ts->deltaT;
  output->result[ix].optional->range.dTime.baseFreq=
    input->ts->f0;
  output->result[ix].optional->range.dTime.phase=0.0;
  output->result[ix].optional->range.dTime.numberChannels=1;
  output->result[ix].optional->range.dTime.numberDetectors=0;
  output->result[ix].optional->range.dTime.channelName=
    (CHAR**)LALCalloc(output->result[ix].optional->range.dTime.numberChannels,sizeof(CHAR*));
  output->result[ix].optional->range.dTime.channelName[0]=
    (CHAR*)LALCalloc(maxMultiDimName,sizeof(CHAR));

  strcpy(output->result[ix].optional->range.dTime.channelName[0], input->ts->name);
  output->result[ix].optional->range.dTime.geometry=NULL;

  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));

  output->result[ix].optional->dimensions[0]=input->ts->data->length; 
  output->result[ix].optional->data.real4=input->ts->data->data;

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALClustersParametersWave(LALStatus *status,
			       LALClustersParametersWaveInput *input,
			       LALSearchOutput *output
			       )
{
  int ix;
  INITSTATUS( status, "LALClustersParametersWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;
  
  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,"Dimensions and fractions");
  strcpy(output->result[ix].optional->units,"none");
  strcpy(output->result[ix].optional->comment, input->comment);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=real_4;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=16; 
  output->result[ix].optional->data.real4=
    (REAL4*)LALCalloc(output->result[ix].optional->dimensions[0], sizeof(REAL4));

  output->result[ix].optional->data.real4[0]=
    (REAL4)input->w1->pMaskCount;
  output->result[ix].optional->data.real4[1]=
    (REAL4)input->w2->pMaskCount;
  output->result[ix].optional->data.real4[2]=
    (REAL4)input->w1->clusterCount;
  output->result[ix].optional->data.real4[3]=
    (REAL4)input->w2->clusterCount;
  output->result[ix].optional->data.real4[4]=
    input->w1->nonZeroFractionAfterPercentile;
  output->result[ix].optional->data.real4[5]=
    input->w2->nonZeroFractionAfterPercentile;
  output->result[ix].optional->data.real4[6]=
    input->w1->nonZeroFractionAfterCoincidence;
  output->result[ix].optional->data.real4[7]=
    input->w2->nonZeroFractionAfterCoincidence;
  output->result[ix].optional->data.real4[8]=
    input->w1->nonZeroFractionAfterSetMask;
  output->result[ix].optional->data.real4[9]=
    input->w2->nonZeroFractionAfterSetMask;
  output->result[ix].optional->data.real4[10]=
    input->w1->nonZeroFractionAfterClustering;
  output->result[ix].optional->data.real4[11]=
    input->w2->nonZeroFractionAfterClustering;
  output->result[ix].optional->data.real4[12]=
    input->w1->nonZeroFractionAfterCuts;
  output->result[ix].optional->data.real4[13]=
    input->w2->nonZeroFractionAfterCuts;
  output->result[ix].optional->data.real4[14]=
    input->w1->nonZeroFractionAfterVetoes;
  output->result[ix].optional->data.real4[15]=
    input->w2->nonZeroFractionAfterVetoes;


  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );

}

void LALClustersMediansWave(LALStatus *status,
			    LALClustersMediansWaveInput *input,
			    LALSearchOutput *output
			    )
{
  int ix;
  INITSTATUS( status, "LALClustersMediansWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;
  
  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,"medians");
  strcpy(output->result[ix].optional->units,"none");
  strcpy(output->result[ix].optional->comment, input->comment);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=real_4;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=1<<input->w->wavelet->level; 
  output->result[ix].optional->data.real4=
    (REAL4*)LALCalloc(output->result[ix].optional->dimensions[0], sizeof(REAL4));

  output->result[ix].optional->data.real4=input->w->medians;

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );

}


void LALClustersVolumesWave(LALStatus *status,
			    LALClustersVolumesWaveInput *input,
			    LALSearchOutput *output
			    )
{
  int ix;
  INITSTATUS( status, "LALClustersVolumesWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;


  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  strcpy(output->result[ix].optional->comment,input->comment);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->clusterCount; 
  output->result[ix].optional->data.int4u=input->w->volumes;

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void LALClustersListWave(LALStatus *status,
			 LALClustersListWaveInput *input,
			 LALSearchOutput *output
			 )
{
  int ix;
  int i,j;
  int counter;
  INITSTATUS( status, "LALClustersListWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  strcpy(output->result[ix].optional->comment,input->comment);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=0;
  
  for(i=0;i<input->w->clusterCount;i++)
    {
      output->result[ix].optional->dimensions[0]+=input->w->volumes[i];
    }

  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  counter=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      for(j=0;j<input->w->volumes[i];j++)
	{
	  output->result[ix].optional->data.int4u[counter++]=
	    input->w->cList[i][j];
	}
    }

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void LALClustersPMaskWave(LALStatus *status,
			  LALClustersPMaskWaveInput *input,
			  LALSearchOutput *output
			 )
{
  int ix;
  int i,j;
  int counter;
  CHAR tmp[LALNameLength];
  INITSTATUS( status, "LALClustersPMASKWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=(multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": time");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.int4u[i]=input->w->pMask[i]->time;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": frequency");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.int4u[i]=
	input->w->pMask[i]->frequency;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": clusterID");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.int4u[i]=
	input->w->pMask[i]->clusterID;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": core");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.int4u[i]=
	input->w->pMask[i]->core ? 1:0;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": neighborsCount");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.int4u[i]=
	input->w->pMask[i]->neighborsCount;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": amplitude");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=real_4;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.real4=
    (REAL4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(REAL4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.real4[i]=
	input->w->pMask[i]->amplitude;
    }

  (*input->counter)++;

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": original amplitude");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=real_4;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));
  output->result[ix].optional->dimensions[0]=input->w->pMaskCount;
  output->result[ix].optional->data.real4=
    (REAL4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(REAL4));

  for(i=0;i<output->result[ix].optional->dimensions[0];i++)
    {
      output->result[ix].optional->data.real4[i]=
	input->w->pMask[i]->amplitudeOriginal;
    }

  (*input->counter)++;


  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;

  output->result[ix].results=NULL;
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  output->result[ix].optional=
    (multiDimData*)LALMalloc(sizeof(multiDimData));
  strcpy(output->result[ix].optional->name,input->w->wavelet->data->name);
  strcpy(output->result[ix].optional->units,"none");
  tmp[0]='\0';
  strcpy(tmp,input->comment);
  strcat(tmp,": neighbors");
  strcpy(output->result[ix].optional->comment,tmp);
  output->result[ix].optional->space=timeD;
  output->result[ix].optional->type=int_4u;
  output->result[ix].optional->history=NULL;
  output->result[ix].optional->range.dTime=
    input->my->input->sequences[0].range.dTime;
  output->result[ix].optional->numberDimensions=1;
  output->result[ix].optional->dimensions=
    (UINT4*)LALCalloc(output->result[ix].optional->numberDimensions,sizeof(UINT4));

  output->result[ix].optional->dimensions[0]=0;
  for(i=0;i<input->w->pMaskCount;i++)
    {
      output->result[ix].optional->dimensions[0]+=input->w->pMask[i]->neighborsCount;
    }

  output->result[ix].optional->data.int4u=
    (UINT4*)LALCalloc(output->result[ix].optional->dimensions[0],sizeof(UINT4));

  counter=0;
  for(i=0;i<input->w->pMaskCount;i++)
    {
      for(j=0;j<input->w->pMask[i]->neighborsCount;j++)
	{
	  output->result[ix].optional->data.int4u[counter++]=
	    input->w->pMask[i]->neighbors[j];
	}
    }

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void LALBlobsDBWave(LALStatus *status,
		    LALBlobsDBWaveInput *input,
		    LALSearchOutput *output
		    )
{
  int ix;
  LALBLOB blob;
  dataBase *dbase=NULL;
  int i,j;
  CHAR tmp[LALNameLength];

  INITSTATUS( status, "LALBlobsDBWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  ix=*input->counter;

  output->result[ix].search=burst;
  output->result[ix].significant=1;
  output->result[ix].results=NULL;
  
  output->result[ix].states=NULL;
  output->result[ix].indexNumber=ix;

  dbase=NULL;
  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst_mime","mimetype");
  CHECKSTATUSPTR(status);
  output->result[ix].results=dbase;
  blob.data=input->comment;
  blob.size=strlen(input->comment);
  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
		       &blob, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst_mime","t_size");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{
	  dbase->rows.int4s[j++]=
	    input->w->blobs[i].time_width;
	}
    }

  LALCreateNextDBEntry(status->statusPtr, &dbase, 
		       "waveburst_mime","f_size");
  CHECKSTATUSPTR(status);
  LALCreateDBEntryData(status->statusPtr, dbase, 
		       int_4s, input->w->clusterCountFinal);
  CHECKSTATUSPTR(status);
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{      
	  dbase->rows.int4s[j++]=
	    input->w->blobs[i].freq_width;
	}
    }
  j=0;
  for(i=0;i<input->w->clusterCount;i++)
    {
      if(input->w->sCuts[i]==1)
	{      
	  LALCreateNextDBEntry(status->statusPtr, &dbase, 
			       "waveburst_mime","cluster_o");
	  CHECKSTATUSPTR(status);
	  blob.data=input->w->blobs[i].oBlob;
	  blob.size=
	    sizeof(REAL4) * input->w->blobs[i].freq_width * input->w->blobs[i].time_width;
	  LALCreateDBEntryUBLOB(status->statusPtr, dbase, 
				&blob, 1);
	  CHECKSTATUSPTR(status);
	  
	  LALCreateNextDBEntry(status->statusPtr, &dbase, 
			       "waveburst_mime","cluster_p");
	  CHECKSTATUSPTR(status);
	  blob.data=input->w->blobs[i].pBlob;
	  blob.size=
	    sizeof(REAL4) * input->w->blobs[i].freq_width * input->w->blobs[i].time_width;
	  LALCreateDBEntryUBLOB(status->statusPtr, dbase, 
				&blob, 1);
	  CHECKSTATUSPTR(status);
	  
	  LALCreateNextDBEntry(status->statusPtr, &dbase, 
			       "waveburst_mime","event_id");
	  CHECKSTATUSPTR(status);
	  sprintf(tmp,"waveburstgroup:event_id:%d",j + input->startEventID); j++;
	  blob.data=tmp;
	  blob.size=strlen(tmp);
	  LALCreateDBEntryBLOB(status->statusPtr, dbase, 
			       &blob, 1);
	  CHECKSTATUSPTR(status);
	}
    }

/*    LALCreateNextDBEntry(status->statusPtr, &dbase,  */
/*  		       "waveburst_mime","process_id"); */
/*    CHECKSTATUSPTR(status); */
/*    blob.data="processgroup:process:process_id:0"; */
/*    blob.size=strlen(blob.data); */
/*    LALCreateDBEntryBLOB(status->statusPtr, dbase,  */
/*  		       &blob, input->w->clusterCount); */
/*    CHECKSTATUSPTR(status); */

  (*input->counter)++;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void LALSelectWave2(LALStatus *status,
		   WaveParams *my)
{
  INT4 i;

  INITSTATUS( status, "LALSelectWave2", OUTPUTC );
  ATTATCHSTATUSPTR( status );
  
  my->oneOutCluster->w->clusterCountFinal=0;
  my->twoOutCluster->w->clusterCountFinal=0;
  
  for(i=0;i<my->oneOutCluster->w->clusterCount;i++)
    {
      if(my->oneOutCluster->w->likelihood[i] <= my->maxLikelihood && 
	 my->oneOutCluster->w->likelihood[i] >= my->minLikelihood &&

	 my->oneOutCluster->w->power[i] <= my->maxPower && 
	 my->oneOutCluster->w->power[i] >= my->minPower &&

	 my->oneOutCluster->w->power[i]*my->oneOutCluster->w->coreSize[i] <= my->maxEnergy && 
	 my->oneOutCluster->w->power[i]*my->oneOutCluster->w->coreSize[i] >= my->minEnergy &&
	 
	 my->oneOutCluster->w->coreSize[i] <= my->maxClusterSize &&
	 my->oneOutCluster->w->coreSize[i] >= my->minClusterSize
	 )
	{
	  my->oneOutCluster->w->sCuts[i]=1;
	  my->oneOutCluster->w->clusterCountFinal++;
	}
      else
	{
	  my->oneOutCluster->w->sCuts[i]=0;
	}
    }

  for(i=0;i<my->twoOutCluster->w->clusterCount;i++)
    {
      if(my->twoOutCluster->w->likelihood[i] <= my->maxLikelihood && 
	 my->twoOutCluster->w->likelihood[i] >= my->minLikelihood &&

	 my->twoOutCluster->w->power[i] <= my->maxPower && 
	 my->twoOutCluster->w->power[i] >= my->minPower &&

	 my->twoOutCluster->w->power[i]*my->twoOutCluster->w->coreSize[i] <= my->maxEnergy && 
	 my->twoOutCluster->w->power[i]*my->twoOutCluster->w->coreSize[i] >= my->minEnergy &&
	 
	 my->twoOutCluster->w->coreSize[i] <= my->maxClusterSize &&
	 my->twoOutCluster->w->coreSize[i] >= my->minClusterSize 
	 )
	{
	  my->twoOutCluster->w->sCuts[i]=1;
	  my->twoOutCluster->w->clusterCountFinal++;
	}
      else
	{
	  my->twoOutCluster->w->sCuts[i]=0;
	}
    }

  if(my->usePixelSwapOne || my->usePixelSwapTwo)
    {
      my->oneOutClusterPS->w->clusterCountFinal=0;
      my->twoOutClusterPS->w->clusterCountFinal=0;

      for(i=0;i<my->oneOutClusterPS->w->clusterCount;i++)
	{
	  if(my->oneOutClusterPS->w->likelihood[i] <= my->maxLikelihood && 
	     my->oneOutClusterPS->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->oneOutClusterPS->w->power[i] <= my->maxPower && 
	     my->oneOutClusterPS->w->power[i] >= my->minPower &&
	     
	     my->oneOutClusterPS->w->power[i]*my->oneOutClusterPS->w->coreSize[i] <= my->maxEnergy && 
	     my->oneOutClusterPS->w->power[i]*my->oneOutClusterPS->w->coreSize[i] >= my->minEnergy &&
	     
	     my->oneOutClusterPS->w->coreSize[i] <= my->maxClusterSize &&
	     my->oneOutClusterPS->w->coreSize[i] >= my->minClusterSize
	     )
	    {
	      my->oneOutClusterPS->w->sCuts[i]=1;
	      my->oneOutClusterPS->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->oneOutClusterPS->w->sCuts[i]=0;
	    } 
	}

      for(i=0;i<my->twoOutClusterPS->w->clusterCount;i++)
	{
	  if(my->twoOutClusterPS->w->likelihood[i] <= my->maxLikelihood && 
	     my->twoOutClusterPS->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->twoOutClusterPS->w->power[i] <= my->maxPower && 
	     my->twoOutClusterPS->w->power[i] >= my->minPower &&
	     
	     my->twoOutClusterPS->w->power[i]*my->twoOutClusterPS->w->coreSize[i] <= my->maxEnergy && 
	     my->twoOutClusterPS->w->power[i]*my->twoOutClusterPS->w->coreSize[i] >= my->minEnergy &&
	     
	     my->twoOutClusterPS->w->coreSize[i] <= my->maxClusterSize &&
	     my->twoOutClusterPS->w->coreSize[i] >= my->minClusterSize 
	     )
	    {
	      my->twoOutClusterPS->w->sCuts[i]=1;
	      my->twoOutClusterPS->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->twoOutClusterPS->w->sCuts[i]=0;
	    } 
	}
    }

  if(my->usePixelMixer)
    {
      my->oneOutClusterPM->w->clusterCountFinal=0;
      my->twoOutClusterPM->w->clusterCountFinal=0;
  
      for(i=0;i<my->oneOutClusterPM->w->clusterCount;i++)
	{
	  if(my->oneOutClusterPM->w->likelihood[i] <= my->maxLikelihood && 
	     my->oneOutClusterPM->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->oneOutClusterPM->w->power[i] <= my->maxPower && 
	     my->oneOutClusterPM->w->power[i] >= my->minPower &&
	     
	     my->oneOutClusterPM->w->power[i]*my->oneOutClusterPM->w->coreSize[i] <= my->maxEnergy && 
	     my->oneOutClusterPM->w->power[i]*my->oneOutClusterPM->w->coreSize[i] >= my->minEnergy &&
	     
	     my->oneOutClusterPM->w->coreSize[i] <= my->maxClusterSize &&
	     my->oneOutClusterPM->w->coreSize[i] >= my->minClusterSize
	     )
	    {
	      my->oneOutClusterPM->w->sCuts[i]=1;
	      my->oneOutClusterPM->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->oneOutClusterPM->w->sCuts[i]=0;
	    } 
	}

      for(i=0;i<my->twoOutClusterPM->w->clusterCount;i++)
	{
	  if(my->twoOutClusterPM->w->likelihood[i] <= my->maxLikelihood && 
	     my->twoOutClusterPM->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->twoOutClusterPM->w->power[i] <= my->maxPower && 
	     my->twoOutClusterPM->w->power[i] >= my->minPower &&
	     
	     my->twoOutClusterPM->w->power[i]*my->twoOutClusterPM->w->coreSize[i] <= my->maxEnergy && 
	     my->twoOutClusterPM->w->power[i]*my->twoOutClusterPM->w->coreSize[i] >= my->minEnergy &&
	     
	     my->twoOutClusterPM->w->coreSize[i] <= my->maxClusterSize &&
	     my->twoOutClusterPM->w->coreSize[i] >= my->minClusterSize 
	     )
	    {
	      my->twoOutClusterPM->w->sCuts[i]=1;
	      my->twoOutClusterPM->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->twoOutClusterPM->w->sCuts[i]=0;
	    } 
	}
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );

}


void LALSelectWave1(LALStatus *status,
		   WaveParams *my)
{
  INT4 i;

  INITSTATUS( status, "LALSelectWave1", OUTPUTC );
  ATTATCHSTATUSPTR( status );
  
  my->oneOutCluster->w->clusterCountFinal=0;
  my->twoOutCluster->w->clusterCountFinal=0;
  
  for(i=0;i<my->oneOutCluster->w->clusterCount;i++)
    {
      if(my->oneOutCluster->w->likelihood[i] <= my->maxLikelihood && 
	 my->oneOutCluster->w->likelihood[i] >= my->minLikelihood &&
	 my->twoOutCluster->w->likelihood[i] <= my->maxLikelihood && 
	 my->twoOutCluster->w->likelihood[i] >= my->minLikelihood &&

	 my->oneOutCluster->w->power[i] <= my->maxPower && 
	 my->oneOutCluster->w->power[i] >= my->minPower &&
	 my->twoOutCluster->w->power[i] <= my->maxPower && 
	 my->twoOutCluster->w->power[i] >= my->minPower &&

	 my->oneOutCluster->w->power[i]*my->oneOutCluster->w->coreSize[i] <= my->maxEnergy && 
	 my->oneOutCluster->w->power[i]*my->oneOutCluster->w->coreSize[i] >= my->minEnergy &&
	 my->twoOutCluster->w->power[i]*my->twoOutCluster->w->coreSize[i] <= my->maxEnergy && 
	 my->twoOutCluster->w->power[i]*my->twoOutCluster->w->coreSize[i] >= my->minEnergy &&
	 
	 my->oneOutCluster->w->coreSize[i] <= my->maxClusterSize &&
	 my->oneOutCluster->w->coreSize[i] >= my->minClusterSize &&
	 my->twoOutCluster->w->coreSize[i] <= my->maxClusterSize &&
	 my->twoOutCluster->w->coreSize[i] >= my->minClusterSize 
	 )
	{
	  my->oneOutCluster->w->sCuts[i]=1;
	  my->twoOutCluster->w->sCuts[i]=1;
	  my->oneOutCluster->w->clusterCountFinal++;
	  my->twoOutCluster->w->clusterCountFinal++;
	}
      else
	{
	  my->oneOutCluster->w->sCuts[i]=0;
	  my->twoOutCluster->w->sCuts[i]=0;
	}
	 
    }

  if(my->usePixelSwapOne || my->usePixelSwapTwo)
    {
      my->oneOutClusterPS->w->clusterCountFinal=0;
      my->twoOutClusterPS->w->clusterCountFinal=0;
  
      for(i=0;i<my->oneOutClusterPS->w->clusterCount;i++)
	{
	  if(my->oneOutClusterPS->w->likelihood[i] <= my->maxLikelihood && 
	     my->oneOutClusterPS->w->likelihood[i] >= my->minLikelihood &&
	     my->twoOutClusterPS->w->likelihood[i] <= my->maxLikelihood && 
	     my->twoOutClusterPS->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->oneOutClusterPS->w->power[i] <= my->maxPower && 
	     my->oneOutClusterPS->w->power[i] >= my->minPower &&
	     my->twoOutClusterPS->w->power[i] <= my->maxPower && 
	     my->twoOutClusterPS->w->power[i] >= my->minPower &&
	     
	     my->oneOutClusterPS->w->power[i]*my->oneOutClusterPS->w->coreSize[i] <= my->maxEnergy && 
	     my->oneOutClusterPS->w->power[i]*my->oneOutClusterPS->w->coreSize[i] >= my->minEnergy &&
	     my->twoOutClusterPS->w->power[i]*my->twoOutClusterPS->w->coreSize[i] <= my->maxEnergy && 
	     my->twoOutClusterPS->w->power[i]*my->twoOutClusterPS->w->coreSize[i] >= my->minEnergy &&
	     
	     my->oneOutClusterPS->w->coreSize[i] <= my->maxClusterSize &&
	     my->oneOutClusterPS->w->coreSize[i] >= my->minClusterSize &&
	     my->twoOutClusterPS->w->coreSize[i] <= my->maxClusterSize &&
	     my->twoOutClusterPS->w->coreSize[i] >= my->minClusterSize 
	     )
	    {
	      my->oneOutClusterPS->w->sCuts[i]=1;
	      my->twoOutClusterPS->w->sCuts[i]=1;
	      my->oneOutClusterPS->w->clusterCountFinal++;
	      my->twoOutClusterPS->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->oneOutClusterPS->w->sCuts[i]=0;
	      my->twoOutClusterPS->w->sCuts[i]=0;
	    } 
	}
    }

  if(my->usePixelMixer)
    {
      my->oneOutClusterPM->w->clusterCountFinal=0;
      my->twoOutClusterPM->w->clusterCountFinal=0;
  
      for(i=0;i<my->oneOutClusterPM->w->clusterCount;i++)
	{
	  if(my->oneOutClusterPM->w->likelihood[i] <= my->maxLikelihood && 
	     my->oneOutClusterPM->w->likelihood[i] >= my->minLikelihood &&
	     my->twoOutClusterPM->w->likelihood[i] <= my->maxLikelihood && 
	     my->twoOutClusterPM->w->likelihood[i] >= my->minLikelihood &&
	     
	     my->oneOutClusterPM->w->power[i] <= my->maxPower && 
	     my->oneOutClusterPM->w->power[i] >= my->minPower &&
	     my->twoOutClusterPM->w->power[i] <= my->maxPower && 
	     my->twoOutClusterPM->w->power[i] >= my->minPower &&
	     
	     my->oneOutClusterPM->w->power[i]*my->oneOutClusterPM->w->coreSize[i] <= my->maxEnergy && 
	     my->oneOutClusterPM->w->power[i]*my->oneOutClusterPM->w->coreSize[i] >= my->minEnergy &&
	     my->twoOutClusterPM->w->power[i]*my->twoOutClusterPM->w->coreSize[i] <= my->maxEnergy && 
	     my->twoOutClusterPM->w->power[i]*my->twoOutClusterPM->w->coreSize[i] >= my->minEnergy &&
	     
	     my->oneOutClusterPM->w->coreSize[i] <= my->maxClusterSize &&
	     my->oneOutClusterPM->w->coreSize[i] >= my->minClusterSize &&
	     my->twoOutClusterPM->w->coreSize[i] <= my->maxClusterSize &&
	     my->twoOutClusterPM->w->coreSize[i] >= my->minClusterSize 
	     )
	    {
	      my->oneOutClusterPM->w->sCuts[i]=1;
	      my->twoOutClusterPM->w->sCuts[i]=1;
	      my->oneOutClusterPM->w->clusterCountFinal++;
	      my->twoOutClusterPM->w->clusterCountFinal++;
	    }
	  else
	    {
	      my->oneOutClusterPM->w->sCuts[i]=0;
	      my->twoOutClusterPM->w->sCuts[i]=0;
	    } 
	}
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );

}

void LALSelectWave(LALStatus *status,
		   WaveParams *my)
{
  INITSTATUS( status, "LALSelectWave", OUTPUTC );
  ATTATCHSTATUSPTR( status );

  if(my->coincidenceLevel<0)
    {
      LALSelectWave2(status->statusPtr,my);
    }
  else
    {
      LALSelectWave1(status->statusPtr,my);
    }
  CHECKSTATUSPTR(status);
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
