/*************** <lalVerbatim file="SearchCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Search.c,v 1.10 2003/05/27 17:40:24 igor Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <lal/Random.h>
#include <Wave.h>
#include <BuildDB.h>

NRCSID(SEARCHC, "$Id" );

void LALApplySearch(LALStatus *status,
		    LALSearchOutput *output,
		    LALSearchInput *input,
		    LALApplySearchParams *params
		    )
{

  InputPercentileWavelet oneInPT;
  InputPercentileWavelet twoInPT;

  InputPixelSwapWavelet oneInPS;
  InputPixelSwapWavelet twoInPS;

  InputPixelMixerWavelet oneInPM;
  InputPixelMixerWavelet twoInPM;

  InputCoincidenceWavelet coincidenceIn;

  InputClusterWavelet oneInCluster;
  InputClusterWavelet twoInCluster;

  InputReuseClusterWavelet twoInReuseCluster;

  RandomParams *rparams;

  WaveParams *my=(WaveParams*)params->searchParams;

  INITSTATUS( status, "LALApplySearch", SEARCHC );
  ATTATCHSTATUSPTR( status );

  /* Computations */

  switch(my->ncalls)
    {
    case 0:
      oneInPT.in=my->oneIn;
      oneInPT.nonZeroFraction=my->nonZeroFraction;
      LALPercentileWavelet(status->statusPtr, &my->oneOutPT, &oneInPT);
      CHECKSTATUSPTR(status);
      my->ncalls++;
      output->fracRemaining=0.9;
      output->notFinished=1;
      break;
    case 1:
      twoInPT.in=my->twoIn;
      twoInPT.nonZeroFraction=my->nonZeroFraction;
      LALPercentileWavelet(status->statusPtr, &my->twoOutPT, &twoInPT);
      CHECKSTATUSPTR(status);
      my->ncalls++;
      output->fracRemaining=0.8;
      output->notFinished=1;
      break;
    case 2:
      coincidenceIn.timeWindowPixels=my->timeWindowPixels;
      coincidenceIn.freqWindowPixels=my->freqWindowPixels;
      coincidenceIn.coincidenceLevel=my->coincidenceLevel;
      coincidenceIn.minAmp4SinglePixels=my->minAmp4SinglePixels;
      coincidenceIn.minAmp4ClusterExtension=my->minAmp4ClusterExtension;
      coincidenceIn.one=my->oneOutPT->out;
      coincidenceIn.two=my->twoOutPT->out;      
      coincidenceIn.one->original=my->oneIn;
      coincidenceIn.two->original=my->twoIn;
      LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOut, &coincidenceIn);
      CHECKSTATUSPTR(status);

      if(my->usePixelSwapOne && !my->usePixelSwapTwo)
	{
	  oneInPS.in=my->oneOutPT->out;
	  oneInPS.in->original=my->oneIn;
	  LALPixelSwapWavelet(status->statusPtr, &my->oneOutPS, &oneInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPS->out;
	  coincidenceIn.two=my->twoOutPT->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);
	}

      if(my->usePixelSwapTwo && !my->usePixelSwapOne)
	{
	  twoInPS.in=my->twoOutPT->out;
	  twoInPS.in->original=my->twoIn;

	  LALPixelSwapWavelet(status->statusPtr, &my->twoOutPS, &twoInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPT->out;
	  coincidenceIn.two=my->twoOutPS->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);
	}
      if(my->usePixelSwapOne && my->usePixelSwapTwo)
	{
	  oneInPS.in=my->oneOutPT->out;
	  oneInPS.in->original=my->oneIn;
	  LALPixelSwapWavelet(status->statusPtr, &my->oneOutPS, &oneInPS);
	  CHECKSTATUSPTR(status);
	  twoInPS.in=my->twoOutPT->out;
	  twoInPS.in->original=my->twoIn;
	  LALPixelSwapWavelet(status->statusPtr, &my->twoOutPS, &twoInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPS->out;
	  coincidenceIn.two=my->twoOutPS->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);	
	}
      if(my->usePixelMixer)
	{
	  LALCreateRandomParams(status->statusPtr,&rparams,my->seed);
	  CHECKSTATUSPTR(status);

	  oneInPM.rparams=rparams;
	  oneInPM.in=my->oneOutPT->out;
	  oneInPM.in->original=my->oneIn;
	  LALPixelMixerWavelet(status->statusPtr, &my->oneOutPM, &oneInPM);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPM->out;

	  twoInPM.rparams=rparams;
	  twoInPM.in=my->twoOutPT->out;
	  twoInPM.in->original=my->twoIn;
	  LALPixelMixerWavelet(status->statusPtr, &my->twoOutPM, &twoInPM);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.two=my->twoOutPM->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPM, &coincidenceIn);
	  CHECKSTATUSPTR(status);	

	  LALDestroyRandomParams(status->statusPtr,&rparams);
	  CHECKSTATUSPTR(status);
	}
      my->ncalls++;
      output->fracRemaining=0.7;
      output->notFinished=1;
      break;
    case 3:

      oneInCluster.w=my->coincidenceOut->one;
      oneInCluster.aura=my->aura;
      oneInCluster.minClusterSize=my->minClusterSize;
      oneInCluster.maxClusterSize=my->maxClusterSize;

      oneInCluster.w->noise_rms_flag=my->noise_rms_flag;
      oneInCluster.w->psd=my->onePSD;
      oneInCluster.w->calibration_max_freq=my->calibration_max_freq;

      /*
      if(my->noise_rms_flag)
	{
	  printf("my->PSD[0]=%f my->PSD[1]=%f\n",my->onePSD->data->data[0],my->onePSD->data->data[1]);fflush(stdout);
	}
      printf("HERE?\n");fflush(stdout);
      */

      LALClusterWavelet(status->statusPtr, &my->oneOutCluster, &oneInCluster);
      CHECKSTATUSPTR(status);

      /*
      printf("THERE?\n");fflush(stdout);
      */

      if(my->usePixelSwapOne || my->usePixelSwapTwo)
	{
	  oneInCluster.w=my->coincidenceOutPS->one;

	  oneInCluster.w->noise_rms_flag=my->noise_rms_flag;
	  oneInCluster.w->psd=my->onePSD;
	  oneInCluster.w->calibration_max_freq=my->calibration_max_freq;

	  LALClusterWavelet(status->statusPtr, &my->oneOutClusterPS, &oneInCluster);
	  CHECKSTATUSPTR(status);
	}
      if(my->usePixelMixer)
	{
	  oneInCluster.w=my->coincidenceOutPM->one;

          oneInCluster.w->noise_rms_flag=my->noise_rms_flag;
          oneInCluster.w->psd=my->onePSD;
	  oneInCluster.w->calibration_max_freq=my->calibration_max_freq;

	  LALClusterWavelet(status->statusPtr, &my->oneOutClusterPM, &oneInCluster);
	  CHECKSTATUSPTR(status);	  
	}

      my->ncalls++;
      output->fracRemaining=0.6;
      output->notFinished=1;
      /*      printf("At the end of case 3\n");fflush(stdout); */
      break;
    case 4:

      if(!my->reuseCluster)
	{
	  twoInCluster.w=my->coincidenceOut->two;
	  twoInCluster.aura=my->aura;
	  twoInCluster.minClusterSize=my->minClusterSize;
	  twoInCluster.maxClusterSize=my->maxClusterSize;

          twoInCluster.w->noise_rms_flag=my->noise_rms_flag;
          twoInCluster.w->psd=my->twoPSD;
	  twoInCluster.w->calibration_max_freq=my->calibration_max_freq;
	  
	  LALClusterWavelet(status->statusPtr, &my->twoOutCluster, &twoInCluster);
	  CHECKSTATUSPTR(status);
	  
	  if(my->usePixelSwapOne || my->usePixelSwapTwo)
	    {
	      twoInCluster.w=my->coincidenceOutPS->two;

	      twoInCluster.w->noise_rms_flag=my->noise_rms_flag;
	      twoInCluster.w->psd=my->twoPSD;
	      twoInCluster.w->calibration_max_freq=my->calibration_max_freq;

	      LALClusterWavelet(status->statusPtr, &my->twoOutClusterPS, &twoInCluster);
	      CHECKSTATUSPTR(status);
	    }
	  if(my->usePixelMixer)
	    {
	      twoInCluster.w=my->coincidenceOutPM->two;

	      twoInCluster.w->noise_rms_flag=my->noise_rms_flag;
	      twoInCluster.w->psd=my->twoPSD;
	      twoInCluster.w->calibration_max_freq=my->calibration_max_freq;

	      LALClusterWavelet(status->statusPtr, &my->twoOutClusterPM, &twoInCluster);
	      CHECKSTATUSPTR(status);	  
	    }
	}
      else
	{
          twoInReuseCluster.w=my->coincidenceOut->two;
	  twoInReuseCluster.another=my->oneOutCluster->w;
          twoInReuseCluster.aura=my->aura;
          twoInReuseCluster.minClusterSize=my->minClusterSize;
	  twoInReuseCluster.maxClusterSize=my->maxClusterSize;

          twoInReuseCluster.w->noise_rms_flag=my->noise_rms_flag;
          twoInReuseCluster.w->psd=my->twoPSD;
	  twoInReuseCluster.w->calibration_max_freq=my->calibration_max_freq;

	  LALReuseClusterWavelet(status->statusPtr, &my->twoOutCluster, &twoInReuseCluster);
          CHECKSTATUSPTR(status);

	  if(my->usePixelSwapOne || my->usePixelSwapTwo)
	    {
	      twoInReuseCluster.w=my->coincidenceOutPS->two;
	      twoInReuseCluster.another=my->oneOutClusterPS->w;

	      twoInReuseCluster.w->noise_rms_flag=my->noise_rms_flag;
	      twoInReuseCluster.w->psd=my->twoPSD;
	      twoInReuseCluster.w->calibration_max_freq=my->calibration_max_freq;

              LALReuseClusterWavelet(status->statusPtr, &my->twoOutClusterPS, &twoInReuseCluster);
              CHECKSTATUSPTR(status);
	    }

          if(my->usePixelMixer)
	    {
	      twoInReuseCluster.w=my->coincidenceOutPM->two;
	      twoInReuseCluster.another=my->oneOutClusterPM->w;

	      twoInReuseCluster.w->noise_rms_flag=my->noise_rms_flag;
	      twoInReuseCluster.w->psd=my->twoPSD;
	      twoInReuseCluster.w->calibration_max_freq=my->calibration_max_freq;

              LALReuseClusterWavelet(status->statusPtr, &my->twoOutClusterPM, &twoInReuseCluster);
              CHECKSTATUSPTR(status);
	    }

	}

      my->ncalls++;
      output->fracRemaining=0.5;      
      output->notFinished=1;
      break;
    case 5:
      my->input=input;
      LALBuildOutputWave(status->statusPtr,my, output);
      CHECKSTATUSPTR(status);
      output->notFinished=0;
      my->ncalls++;
      output->fracRemaining=0.0;
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
